#include <pmaps.h>
#include "addrslv.h"
#include <glib.h>

#define SUCCESSFUL 77

int
main(int argc, char ** argv)
{
  PMaps * pmaps;
  pid_t pid;
  AddrslvSymbolInfo* symbol_info;
  PMapsEntry * entry;
  
  gpointer p 	= main;
  char * p_name = "main";
  GError * err = NULL;


  pid = getpid();
  addrslv_init();
  pmaps = pmaps_new(pid, &err);
  if (pmaps == NULL)
    {
      g_warning ("Creating a PMaps is failed: %s", 
		 err->message);
      g_error_free(err);
      return 1;
    }

  /****
   * Resolve
   */  
  symbol_info = addrslv_resolve(pmaps, GPOINTER_TO_INT(p), &err);
  if (symbol_info)
    {
      addrslv_symbolinfo_ref(symbol_info);
      g_message("[before]%p points %s(at %s:%u)", p,
		symbol_info->funcname, 
		(symbol_info->srcfile_quark == (GQuark)0)
		? "???"
		: g_quark_to_string(symbol_info->srcfile_quark), 
		symbol_info->lineno);
      addrslv_symbolinfo_unref(symbol_info);
    }
  else if (err != NULL)
    {
      g_warning("Resolve is failed: %s", 
		err->message);
      g_error_free(err);
      return 1;
    }
  else
    {
      g_warning("[before]cannot find infomation for %p", p);
      return 1;
    }

  
  if (pmaps_update(pmaps, &err) == FALSE)
    {
      g_warning ("Update is failed: %s", err->message);
      g_error_free(err);
      return 1;
    }

  err = NULL;
  symbol_info = addrslv_resolve(pmaps, GPOINTER_TO_INT(p), &err);
  if (symbol_info)
    g_message("[after]%p points %s(at %s:%u)", p, 
	      symbol_info->funcname, 
	      (symbol_info->srcfile_quark == (GQuark)0)
	      ? "???"
	      : g_quark_to_string(symbol_info->srcfile_quark), 
	      symbol_info->lineno);
  else if (err != NULL)
    {
      g_warning ("Resolve is failed: %s", err->message);
      g_error_free(err);
      return 1;
    }
  else
    {
      g_warning("[after]cannot find infomation for %p", p);
      return 1;
    }

  /****
   * Revese resolve
   */
  entry = pmaps_find(pmaps, GPOINTER_TO_INT(p));
  if (entry == NULL)
    {
      g_warning("Cannot find PMapsEntry including %p", p);
      return 2;
    }

  symbol_info = addrslv_reverse_resolve  (pmaps, entry, p_name, &err);
  if (err != NULL)
    {
      fprintf (stderr, 
	       "Gettint the symbol info for `%s' is fialed: %s", 
	       p_name,
	       err->message);
      g_error_free(err);
      return 2;
    }
  else if (symbol_info == NULL)
    {
      fprintf (stderr, 
	       "Gettint the symbol info for `%s' is fialed.",
	       p_name);
      return 2;
    }

  g_message("Funcname: %s, Address: 0x%x(expected: %p)",
	    symbol_info->funcname, symbol_info->address, p);

  if (symbol_info->address != GPOINTER_TO_INT(p))
    {
      g_warning("Unexpected result: 0x%x != %p", symbol_info->address, p);
      return 3;
    }
  
  pmaps_unref(pmaps);
  addrslv_fin();
  
  return SUCCESSFUL;
} 

/* arch-tag: 8852a23d-f337-4de4-90b8-b3c20f1dc146
   (do not change this comment) */
