#include "backtrace.h"
#include <pmaps.h>
#include <addrslv.h>
#include <fptreel.h>

#define SUCCESSFUL 77

typedef int (*func)(int);

func baz;
func bar;
func foo;

int baz0(int e)
{
  pid_t pid;
  GSList * list;
  PMaps * pmaps;
  GError * err = NULL;
  gint stage = 0;
  guint address;
  AddrslvSymbolInfo * symbol_info;
  
  pid = getpid();
  addrslv_init();
  pmaps = pmaps_new(pid, &err);
  if (pmaps == NULL)
    {
      g_warning ("Creating a PMaps is failed: %s\n", 
		 err->message);
      g_error_free(err);
      return 1;
    }
  
  list = backtrace();
  while (list) 
    {
      address = fptreel_reelup_debug(list->data, pmaps, &stage, &err);
      if (err)
	{
	  g_warning ("Reeling up the function address is failed: %s",
		     err->message);
	  g_error_free(err);
	  return 2;
	}
      /* g_message("Stage: %d", stage); */
    
      symbol_info = addrslv_resolve(pmaps, address, &err);
      if (err != NULL)
	{
	  g_warning("Resolve is failed: %s\n", 
		    err->message);
	  g_error_free(err);
	  return 3;
	}
      if (symbol_info && symbol_info->funcname)
	g_message("%08x: %s", address, symbol_info->funcname);
      else
	g_message("%08x", address);
      list = list->next;
    }
  return SUCCESSFUL;
}
int bar0(int d)
{
  return baz(d);
}

int foo0(int c)
{
  return bar(c);
}

int
main(int argc, char ** argv)
{
  baz = baz0;
  bar = bar0;
  foo = foo0;

  return foo(argc);
} 

/* arch-tag: bdaa76e1-bdce-4862-ae9c-f50755ac93b0
   (do not change this comment) */
