#include "fptreel.h"
#include <pmaps.h>
#include <addrslv.h>
#include <glib.h>
#include <string.h>

#define SUCCESSFUL 77

int
main(int argc, char ** argv)
{
  PMaps * pmaps;
  pid_t pid;
  AddrslvSymbolInfo* symbol_info;

  gpointer p 	       = addrslv_fin;
  const gchar * p_name = "addrslv_fin";
  guint address;
  GError * err = NULL;

  gint stage = 0;

  pid = getpid();
  addrslv_init();
  pmaps = pmaps_new(pid, &err);
  if (pmaps == NULL)
    {
      g_warning ("Creating a PMaps is failed: %s\n", 
		 err->message);
      g_error_free(err);
      return 1;
    }

  address = fptreel_reelup_debug(p, pmaps, &stage, &err);
  if (err)
    {
      g_warning ("Reeling up the function address is failed: %s",
		 err->message);
      g_error_free(err);
      return 2;
    }
  
  g_message("Stage: %d", stage);
  
  symbol_info = addrslv_resolve(pmaps, address, &err);
  if (err != NULL)
    {
      g_warning("Resolve is failed: %s\n", 
		err->message);
      g_error_free(err);
      return 3;
    }
  else if (symbol_info == NULL)
    {
      g_warning("Resolve is failed.");
      return 3;
    }
  else if (symbol_info->funcname == NULL)
    {
      g_warning("No function name.");
      return 3;
    }
  else if (strcmp(symbol_info->funcname, p_name) != 0)
    {
      g_warning("Wrong function name is returned: %s", 
		symbol_info->funcname);
      return 3;
    }

  g_message("Funcname: %s, Address: 0x%x, Source file: %s, Line: %u",
	    symbol_info->funcname, symbol_info->address, 
	    g_quark_to_string(symbol_info->srcfile_quark),
	    symbol_info->lineno);
    
  pmaps_unref(pmaps);
  addrslv_fin();

  return SUCCESSFUL;
} 

/* arch-tag: 4c022d52-bd50-4d08-8bde-eba119e3be5c
   (do not change this comment) */
