/* gtracer.h --- gtracer common decls

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#ifndef __GTRACER_H__
#define __GTRACER_H__

#include <glib.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define GTRACER_PLUGIN_NAME "gtracer-plugin"
#define GTRACER_PRELOAD_NAME "gtracer-preload"

typedef struct _GTracerPreloadData GTracerPreloadData;
typedef struct _GTracerPreloadOperations GTracerPreloadOperations;

typedef struct _GTracerPluginData GTracerPluginData;
typedef struct _GTracerPluginOperations GTracerPluginOperations;

typedef struct _GTracerLog GTracerLog;
typedef struct _GTracerLogOperations GTracerLogOperations;

/* Controling logging and life cycle */
#define GTRACER_OBJECT_ZOMBIE_KEY "gtracer_object_zombie"

/*  --------------------------------------------------------------------
    Log Constructors
    -------------------------------------------------------------------- */
#define GTRACER_CALLBACK_LOG_NAME "callback"
typedef GTracerLog* (* GTracerCallbackLogNew)   (gpointer instance, 
						 const char * signal, 
						 GCallback handler, 
						 gboolean after);

#define GTRACER_CONNECT_LOG_NAME "connect"
typedef GTracerLog* (* GTracerConnectLogNew)    (gpointer instance, 
						 const char * signal, 
						 GCallback handler, 
						 GConnectFlags  connect_flags, 
						 gulong handler_id,
						 GSList * backtrace);

#define GTRACER_DISCONNECT_LOG_NAME "disconnect"
typedef GTracerLog* (* GTracerDisconnectLogNew) (gpointer instance, 
						 gulong handler_id,
						 GSList * backtrace);

#define GTRACER_EMIT_LOG_NAME "emit"
typedef GTracerLog* (* GTracerEmitLogNew)       (gpointer instance, 
						 guint signal_id, 
						 GQuark quark, 
						 gboolean after,
						 GSList * backtrace);

#define GTRACER_PMAPS_LOG_NAME "pmaps"
typedef GTracerLog* (* GTracerPMapsLogNew)      (const gchar *module_name);


#define GTRACER_PROXY_LOG_NAME "proxy"
typedef GTracerLog* (* GTracerProxyLogNew)      (GTracerLog *substance_log);

#define GTRACER_IDLE_LOG_NAME "idle"
typedef GTracerLog* (* GTracerIdleLogNew)   (GSourceFunc function,
					     GSource* source,
					     GSList * backtrace);

#define GTRACER_TIMEOUT_LOG_NAME "timeout"
typedef GTracerLog* (* GTracerTimeoutLogNew) (gint interval,
					      GSourceFunc function,
					      GSource* source,
					      GSList * backtrace);

#define GTRACER_DROPSRC_LOG_NAME "dropsrc"
typedef GTracerLog* (* GTracerDropsrcLogNew)  (guint tag);

/* --------------------------------------------------------------------
  Used in preload side
  -------------------------------------------------------------------- */
struct _GTracerPluginData 
{
  GTracerPluginOperations *mops;
};

struct _GTracerPluginOperations
{
  void         (* push_log)            (GTracerLog * log);
  gpointer     (* get_log_constructor) (const gchar * log_name);
};

struct _GTracerLog
{
  GTracerLogOperations *lops;
};

struct _GTracerLogOperations 
{
  GTracerLog * (* ref)   (GTracerLog * log);
  void         (* unref) (GTracerLog * log);
};

/* --------------------------------------------------------------------
  Used in plugin side
  -------------------------------------------------------------------- */
struct _GTracerPreloadData
{
  GTracerPreloadOperations *pops;
};

struct _GTracerPreloadOperations
{
  void   (* start_tracing) (void);
};

#endif /* Not def: __GTRACER_H__ */

G_END_DECLS

/* arch-tag: 12c1e78b-bbc6-4210-85ad-1a7980d10c35
   (do not change this comment) */
