/* environ_service.c --- implementation of environment variables manipulator

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "environ_service.h"
#include <ginspector/ginspector-tag.h> 
#include <string.h>
#include <stdlib.h>

extern gchar **environ;

enum
{
  COL_PAIR = 0,
  NUM_COLS
} ;

void
gtracer_environ_service_init (GError** err)
{
  gtracer_environ_viewer_init(err);
}
  
GtkTreeModel*
gtracer_environ_model_new (void)
{
  GtkListStore  *store;

  store = gtk_list_store_new (NUM_COLS, G_TYPE_STRING);
  g_inspector_tag_to_object(G_OBJECT(store));

  gtracer_environ_model_update(GTK_TREE_MODEL(store));
  
  return GTK_TREE_MODEL(store);
}

void
gtracer_environ_model_update(GtkTreeModel* model)
{
  gchar ** tmp;
  
  GtkTreeIter iter;

  gtk_list_store_clear(GTK_LIST_STORE(model));
  
  for(tmp = environ; *tmp; tmp++) 
    {
      gtk_list_store_append (GTK_LIST_STORE(model), &iter);
      gtk_list_store_set (GTK_LIST_STORE(model), &iter, COL_PAIR, *tmp, -1);
    }
}

void
gtracer_environ_model_extract_key (GtkTreeViewColumn *col,
				   GtkCellRenderer   *renderer,
				   GtkTreeModel      *model,
				   GtkTreeIter       *iter,
				   gpointer           user_data)
{
  gchar * pair;
  gchar * tmp;
  
  gtk_tree_model_get(model, iter, COL_PAIR, &pair, -1);

  tmp = strchr(pair, '=');
  if (tmp)
    {
      tmp[0] = '\0';
      g_object_set(renderer, "text", pair, NULL);
      tmp[0] = '=';
    }
  else
    g_object_set(renderer, "text", pair, NULL);
  g_free(pair);			/* MODEL_GET_FREE */
}

void
gtracer_environ_model_extract_value (GtkTreeViewColumn *col,
				     GtkCellRenderer   *renderer,
				     GtkTreeModel      *model,
				     GtkTreeIter       *iter,
				     gpointer           user_data)
{
  gchar * pair;
  gchar * tmp;
  
  gtk_tree_model_get(model, iter, COL_PAIR, &pair, -1);
  
  tmp = strchr(pair, '=');
  if (tmp)
    g_object_set(renderer, "text", tmp + 1, NULL);
  else
    g_object_set(renderer, "text", "", NULL);
  g_free(pair);			/* MODEL_GET_FREE */
}

gint
gtracer_environ_model_comapre_func (GtkTreeModel *model,
				    GtkTreeIter *a,
				    GtkTreeIter *b,
				    gpointer user_data)
{
  gchar * a_pair;
  gchar * b_pair;
  gint comparison;
  
  gtk_tree_model_get(model, a, COL_PAIR, &a_pair, -1);
  gtk_tree_model_get(model, b, COL_PAIR, &b_pair, -1);
  
  comparison = strcmp(a_pair, b_pair);
  
  g_free(a_pair);			/* MODEL_GET_FREE */
  g_free(b_pair);			/* MODEL_GET_FREE */

  return comparison;
}

void
gtracer_environ_model_edit_value    (GtkTreeModel * model, 
				     GtkTreeIter  * iter, 
				     const gchar  * new_text)
{
  gchar * old_text;
  gchar * key;
  gchar * value;
  gchar * tmp;
  
  gtk_tree_model_get(model, iter, COL_PAIR, &old_text, -1);
  tmp = strchr(old_text, '=');
  if (tmp)
    tmp[0] = '\0';
  key 	   = old_text;

  value = g_strdup(new_text);

  /* Here I use setenv. 
     In my environment, the version of glib which includes 
     g_setenv is not available yet.

     NOTE:
     g_setenv returns TRUE or FALSE.
     setenv returns 0 or -1. Take care when replacing the
     function. */
  if (setenv(key, value, 1) == 0)
    {
      g_message ("setenv: %s=%s", key, getenv(key));
      gtracer_environ_model_update(model);
    }
  else
    {
      g_message ("unsetenv: %s=%s", key, value);
      g_free(key);
      g_free(value);
    }
}

/* arch-tag: 3c962707-0596-48c0-9112-5f469e7a572b
   (do not change this comment) */
