/* fd_service.c --- implementation of the list of file descriptors

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "fd_service.h"
#include <ginspector/ginspector-tag.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

enum
{
  COL_FD = 0,
  COL_FILENAME = 1,
  NUM_COLS
} ;

static GDir * proc_self_fd = NULL;

void
gtracer_fd_service_init (GError** err)
{
  GError * tmp_error = NULL;
  
  proc_self_fd = g_dir_open("/proc/self/fd", 0, &tmp_error);
  if (tmp_error)
    {
      g_propagate_error(err, tmp_error);
      return ;
    }
  gtracer_fd_viewer_init(err);
}
  
GtkTreeModel*
gtracer_fd_model_new (void)
{
  GtkListStore  *store;

  store = gtk_list_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_STRING);
  g_inspector_tag_to_object(G_OBJECT(store));

  gtracer_fd_model_update(GTK_TREE_MODEL(store), NULL); /* TODO: Error */
  return GTK_TREE_MODEL(store);
}

void
gtracer_fd_model_update(GtkTreeModel* model, GError ** err)
{
  const gchar* link_from;
  gchar*       link_to;

  GtkTreeIter iter;
  gtk_list_store_clear(GTK_LIST_STORE(model));
  g_dir_rewind(proc_self_fd);

  while ((link_from = g_dir_read_name(proc_self_fd)))
    {
#ifdef HAS_G_FILE_READ_LINK
      {
	GError * tmp_error;
	gchar * tmp_from = g_strdup_printf("/proc/self/fd/%s", link_from);
	link_to = g_file_read_link(tmp_from, &tmp_error);
	g_free(tmp_from);
	if (link_to == NULL)
	  {
	    g_propagate_error(err, tmp_error);
	    return ;
	  }
      }
#else
      {
#define BUFSIZE 1024
	gchar * tmp_from = g_strdup_printf("/proc/self/fd/%s", link_from);
	static gchar tmp_to[BUFSIZE];
	gint length;
	memset(tmp_to, '\0', BUFSIZE);
	length = readlink(tmp_from, tmp_to, BUFSIZE -1);
#undef BUFSIZE	  
	g_free(tmp_from);
	if (length < 0)
	  {
	    g_set_error(err, 
			g_file_error_quark(),
			g_file_error_from_errno(errno),
			"Cannot read the link of /proc/self/fd/%s: %s",
			link_from, g_strerror (errno));
	    return ;
	  }
	link_to = g_strdup(tmp_to);
      }
#endif /* Def: HAS_G_FILE_READ_LINK */	

      gtk_list_store_append (GTK_LIST_STORE(model), &iter);
      gtk_list_store_set (GTK_LIST_STORE(model), &iter, COL_FD, link_from, -1);
      gtk_list_store_set (GTK_LIST_STORE(model), &iter, COL_FILENAME, link_to, -1);
 
      g_free(link_to);
    }
}

void
gtracer_fd_model_extract_fd (GtkTreeViewColumn *col,
			     GtkCellRenderer   *renderer,
			     GtkTreeModel      *model,
			     GtkTreeIter       *iter,
			     gpointer           user_data)
{
  gchar * fd;
  gtk_tree_model_get(model, iter, COL_FD, &fd, -1);
  g_object_set(renderer, "text", fd, NULL);
  g_free(fd);			/* MODEL_GET_FREE */
}

void
gtracer_fd_model_extract_filename (GtkTreeViewColumn *col,
				   GtkCellRenderer   *renderer,
				   GtkTreeModel      *model,
				   GtkTreeIter       *iter,
				   gpointer           user_data)
{
  gchar * fd;
  gtk_tree_model_get(model, iter, COL_FILENAME, &fd, -1);
  g_object_set(renderer, "text", fd, NULL);
  g_free(fd);			/* MODEL_GET_FREE */
}

/* arch-tag: 4a0a58c2-c3b7-46d9-9d87-3becff4e5dbf
   (do not change this comment) */
