/* filter.c --- filter common routines

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "plugin.h"

static GData * gtracer_filters = NULL;

struct _GTracerFilter
{
  GQuark log_type;
  GTracerFilterFunc func;
  GTracerFilterHint hint;
  gpointer data;
  guint ref_count;
};


static GTracerFilter * gtracer_filter_new (GQuark log_type,
					   GTracerFilterFunc func,
					   GTracerFilterHint hint,
					   gpointer data);
static void            gtracer_filter_free (GTracerFilter * filter);


void
gtracer_filters_init(void)
{
  if (gtracer_filters == NULL)
    g_datalist_init(&gtracer_filters);
}

GTracerFilter*
gtracer_filters_find    (GQuark log_type)
{
  return g_datalist_id_get_data(&gtracer_filters, log_type);
}

void
gtracer_filters_install (const gchar * log_name,
			 GTracerFilterFunc filter,
			 GTracerFilterHint hint,
			 gpointer filter_data)
{
  GQuark log_type      = g_quark_from_string(log_name);
  GTracerFilter * view = gtracer_filter_new(log_type, filter, hint, filter_data);
  g_datalist_id_set_data(&gtracer_filters, log_type, view);
}


static GTracerFilter *
gtracer_filter_new (GQuark log_type,
		    GTracerFilterFunc func,
		    GTracerFilterHint hint,
		    gpointer data)
{
  GTracerFilter * filter = g_new(GTracerFilter, 1);

  filter->log_type  = log_type;
  filter->func 	    = func;
  filter->hint 	    = hint;
  filter->data 	    = data;
  filter->ref_count = 1;
  return filter;
}

gboolean
gtracer_filter_filter_log (GTracerFilter* filter, 
			   GTracerLogProtected * log,
			   GTracerProxyLog * proxy_log)
{
  g_return_val_if_fail (log, FALSE);

  if (filter)
    return filter->func(log, filter->data, proxy_log);
  else
    return TRUE;
}

void
gtracer_filter_give_hint  (GTracerFilter* filter,
			   gboolean start)
{
  if (filter == NULL) 
    return ;

  if (filter->hint == NULL)
    return ;
      
  filter->hint (filter->log_type,
		filter->data,
		start);
}

GTracerFilter*
gtracer_filter_ref   (GTracerFilter* filter)
{
  g_return_val_if_fail (filter, NULL);
  filter->ref_count++;
  return filter;
}

void
gtracer_filter_unref (GTracerFilter* filter)
{
  g_return_if_fail (filter);
  filter->ref_count--;
  if (filter->ref_count == 0)
    gtracer_filter_free(filter);
}

static void
gtracer_filter_free (GTracerFilter * filter)
{
  g_return_if_fail (filter);
  g_free(filter);
}

/* arch-tag: 790fc130-50a4-4038-8867-3e93cb6b25ca
   (do not change this comment) */
