/* fqarray.h --- Data structure to store the frequency of events 
                 identified by numbers

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#ifndef __FQ_ARRAY_H__
#define __FQ_ARRAY_H__ 

#include <glib.h>

G_BEGIN_DECLS

typedef struct _FqArray FqArray;
typedef void (* FqArrayUpdateNotify)(gpointer client,
				     guint new_index, 
				     gpointer user_data);

FqArray * fq_array_new(FqArrayUpdateNotify notify,
		       gpointer client);

/* Return index */
guint     fq_array_add     (FqArray * fqa, gpointer user_data);

/* Return new index */
guint     fq_array_incr(FqArray * fqa, guint index); 

/* Return the index of inflection */
guint     fq_array_inflection_index(FqArray * fqa, gfloat rate);

/* Accessor */
gulong    fq_array_frequency (FqArray * fqa, guint index);
guint     fq_array_length    (FqArray * fqa);
gulong    fq_array_diff      (FqArray * fqa, guint index);
gpointer  fq_array_user_data (FqArray * fqa, guint index);

/* For debugging 
   `user_data' must be a strnig or `fq_array_dump' will cause
   a crash. */
void      fq_array_dump(FqArray * fqa);

G_END_DECLS

#endif /* Not def: __FQ_ARRAY_H__ */

/* arch-tag: 724bde98-04ab-4a2e-a42c-ed6424c4c856
   (do not change this comment) */
