/* log.c --- log class common routines

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "plugin.h"

static void gtracer_log_free(GTracerLogProtected * log);

GTracerLog*
gtracer_log_ref        (GTracerLog * log_public)
{
  GTracerLogProtected * log = (GTracerLogProtected *)log_public;

  g_return_val_if_fail (log, NULL);
  /* g_warning("+ %p => %d", log, log->ref_count); */
  g_return_val_if_fail (log->ref_count > 0, NULL);
  log->ref_count++;
  return log_public;
}

void
gtracer_log_unref        (GTracerLog * log_public)
{
  GTracerLogProtected * log = (GTracerLogProtected *)log_public;

  g_return_if_fail (log);
  /* g_warning("- %p => %d", log, log->ref_count); */
  g_return_if_fail (log->ref_count > 0);
  log->ref_count--;
  if (log->ref_count == 0)
    gtracer_log_free(log);
}

const gchar*
gtracer_log_type_name  (GTracerLog * log)
{
  g_return_val_if_fail (log, NULL);
  return g_quark_to_string(((GTracerLogProtected *)log)->log_type);
}

void
gtracer_log_init       (GTracerLogProtected * log, GQuark log_type, GTracerLogProtectedOperations * props)
{
  GTracerLog * log_public = (GTracerLog * )log;
  static GTracerLogOperations lops = {
    .ref   = gtracer_log_ref,
    .unref = gtracer_log_unref,
  };
  
  g_return_if_fail (log);
  g_return_if_fail (log_type);
  g_return_if_fail (props);

  /* public part */
  log_public->lops = &lops;
  
  /* protected part */
  log->log_type  = log_type;
  log->ref_count = 1;
  log->thread 	 = g_thread_self();
  log->props 	 = props;
  g_get_current_time(&log->time_stamp);
}

void
gtracer_log_fin (GTracerLogProtected * log)
{
  /* DO NOTHING NOW */
}

gboolean
gtracer_log_is         (GTracerLogProtected * log, const gchar * log_name)
{
  GQuark log_type;

  g_return_val_if_fail (log_name, FALSE);

  log_type = g_quark_try_string(log_name);

  return (log->log_type == log_type)? TRUE: FALSE;
}

void
gtracer_log_dump (GTracerLogProtected * log)
{
  g_return_if_fail (log);
  g_return_if_fail (log->props);
  g_return_if_fail (log->props->free);
  g_message("<timestamp>%ld</timestamp>", 
	    log->time_stamp.tv_sec - gtracer_epoch.tv_sec);
  log->props->dump(log);
}

static void
gtracer_log_free(GTracerLogProtected * log)
{
  g_return_if_fail (log);
  g_return_if_fail (log->props);
  g_return_if_fail (log->props->free);
  log->props->free(log);
  /* g_warning("o %p => %d", log); */
}

/* arch-tag: 87f5420d-1344-4f3a-ac55-a3ade3de806e
   (do not change this comment) */
