/* plugin.h --- Interface exported from gtracer plugin to services.

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#ifndef __GTRACER_PLUGIN_H__
#define __GTRACER_PLUGIN_H__ 

#include <glib.h>
#include <gtk/gtk.h>
#include "gtracer.h"

G_BEGIN_DECLS

typedef struct _GTracerViewer GTracerViewer;
typedef struct _GTracerFilter GTracerFilter;
typedef struct _GTracerLogProtected GTracerLogProtected;
typedef struct _GTracerLogProtectedOperations GTracerLogProtectedOperations;

typedef struct _GTracerProxyLog GTracerProxyLog;

typedef void     (* GTracerServiceInitializer)   (GError ** err);
typedef void     (* GTracerViewerFunc)           (GTracerLogProtected * log,
						  gpointer viewer_data,
						  GTracerProxyLog * proxy);
typedef void     (* GTracerViewerDock)           (GtkWidget * widget,
						  GQuark log_type,
						  gpointer viewer_data);
typedef gboolean (* GTracerFilterFunc)           (GTracerLogProtected * log,
						  gpointer filter_data,
						  GTracerProxyLog * proxy);
typedef void     (* GTracerFilterHint)           (GQuark log_type,
						  gpointer filter_data,
						  gboolean start);

/* ------------------------------------------------------------------
   Service initializing
   ------------------------------------------------------------------
   New log struct should provide a constructor
   (allocating memory and initializing it). You have to install 
   the constructor to gtracer module's constructor table... [Write more]. 
   ------------------------------------------------------------------ */
void gtracer_services_init           (GError ** err);
void gtracer_register_log_constructor(GQuark log_type, gpointer log_constructor);
void gtracer_make_object_zombie      (GObject * object);

/* Timestamp */
extern GTimeVal gtracer_epoch;

/* ------------------------------------------------------------------
   Log Queue
   ------------------------------------------------------------------ */
void        gtracer_queue_init     (void);
GTracerLog* gtracer_queue_pop_log  (void);
void        gtracer_queue_freeze   (void);
void        gtracer_queue_thaw     (void);

/* Exported to the preload module */
void        gtracer_queue_push_log (GTracerLog * log);

/* ------------------------------------------------------------------
   Log
   ------------------------------------------------------------------ */
struct _GTracerLogProtected
{
  GTracerLog            log_public;
  GQuark                log_type;
  guint                 ref_count;
  GThread              *thread;
  GTimeVal              time_stamp;
  GTracerLogProtectedOperations *props;
};

struct _GTracerLogProtectedOperations
{
  void               (*dump) (GTracerLogProtected * log);
  void               (*free) (GTracerLogProtected * log);
};

/* Implementation of public operations. 
   With these functions, fields of GTracerLogOperations are filled
   by `gtracer_log_init'. */
GTracerLog*   gtracer_log_ref        (GTracerLog * log);
void          gtracer_log_unref      (GTracerLog * log);
const gchar*  gtracer_log_type_name  (GTracerLog * log);

/* Helper for new log struct. */
void          gtracer_log_init       (GTracerLogProtected * log, GQuark log_type, GTracerLogProtectedOperations * lops);
void          gtracer_log_fin        (GTracerLogProtected * log);

/* Wrapper for protected operations. 
   Real implementation is needed in your own log struct. */
void          gtracer_log_dump       (GTracerLogProtected * log);

/* type utils */
gboolean      gtracer_log_is         (GTracerLogProtected * log, const gchar * log_name);

/* gtype utils */
extern GType GTracerLogType;
#define GTRACER_LOG_TYPE_NAME "GTracerLog"

/* ------------------------------------------------------------------
   Viewer
   ------------------------------------------------------------------ */
void              gtracer_viewers_init    (void);
GTracerViewer*    gtracer_viewers_find    (GQuark log_type);
void              gtracer_viewers_install (const gchar * log_name,
					   GTracerViewerFunc viewer_func,
					   GTracerViewerDock viewer_dock,
					   gpointer viewer_data);

void              gtracer_viewer_push_log (GTracerViewer* viewer, 
					   GTracerLogProtected * log,
					   GTracerProxyLog * proxy_log);
GTracerViewer*    gtracer_viewer_ref      (GTracerViewer* viewer);
void              gtracer_viewer_unref    (GTracerViewer* viewer);

void              gtracer_viewer_dock_filter_ui (GTracerViewer* viewer,
						 GtkWidget * filter_ui);

/* ------------------------------------------------------------------
   Filter
   ------------------------------------------------------------------ */
/** GTracerFilterFunc: 
    TRUE stands for acceptance. 
    FALSE stands for rejection.*/
void              gtracer_filters_init      (void);
GTracerFilter*    gtracer_filters_find      (GQuark log_type);
void              gtracer_filters_install   (const gchar * log_name,
					     GTracerFilterFunc filter,
					     GTracerFilterHint hint,
					     gpointer filter_data);
gboolean          gtracer_filter_filter_log (GTracerFilter* filter, 
					     GTracerLogProtected * log,
					     GTracerProxyLog * proxy_log);
void              gtracer_filter_give_hint  (GTracerFilter* filter,
					     gboolean start);
GTracerFilter*    gtracer_filter_ref        (GTracerFilter* filter);
void              gtracer_filter_unref      (GTracerFilter* filter);

/* ------------------------------------------------------------------
   GTracer Main Loop Information 
   Check functions and tags which are owned by the main loop.
   ------------------------------------------------------------------ */
extern gboolean gtracer_check_function(gpointer function);
extern gboolean gtracer_check_tag(guint tag);

G_END_DECLS

#endif /* Not def: __GTRACER_PLUGIN_H__ */

/* arch-tag: aa91b68c-f46a-47f9-8f7e-d09f5243f589
   (do not change this comment) */
