/* proxy_log.c --- A log class wraping another log to manage timestamp

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "proxy_service.h"
#include "service.h"
#include "plugin.h"

static GQuark gtracer_proxy_log_type = (GQuark)0;
struct _GTracerProxyLog
{
  GTracerLogProtected log_protected;
  GTracerLogProtected * substance;
};

static void gtracer_proxy_log_dump (GTracerLogProtected * log_protected);
static void gtracer_proxy_log_free (GTracerLogProtected * log_protected);

void
gtracer_proxy_log_class_init    (GError ** err)
{
  gtracer_proxy_log_type = g_quark_from_static_string(GTRACER_PROXY_LOG_NAME);
  gtracer_register_log_constructor(gtracer_proxy_log_type,
				   gtracer_proxy_log_new);
}

GTracerLog*
gtracer_proxy_log_new (GTracerLog * log)
{
  GTracerProxyLog * proxy_log;
  static GTracerLogProtectedOperations proxy_props = {
    .dump       = gtracer_proxy_log_dump,
    .free       = gtracer_proxy_log_free,
  };
  
  g_return_val_if_fail (log, NULL);
  proxy_log = g_new(GTracerProxyLog, 1);
  gtracer_log_init((GTracerLogProtected *)proxy_log, 
		   gtracer_proxy_log_type, 
		   &proxy_props);
  proxy_log->substance = (GTracerLogProtected *)gtracer_log_ref(log);
  return (GTracerLog*)proxy_log;
}

static void
gtracer_proxy_log_dump (GTracerLogProtected * log_protected)
{
  GTracerProxyLog * proxy_log = (GTracerProxyLog *)log_protected;
  g_return_if_fail (proxy_log);
  g_message("<proxy>");
  gtracer_log_dump(proxy_log->substance);
  g_message("</proxy>");
}

static void
gtracer_proxy_log_free (GTracerLogProtected * log_protected)
{
  GTracerProxyLog * proxy_log = (GTracerProxyLog *)log_protected;

  gtracer_log_fin(log_protected);
  if (proxy_log->substance) 
    gtracer_log_unref((GTracerLog*)(proxy_log->substance));
  proxy_log->substance = NULL;
  g_free(proxy_log);
}

GTracerLogProtected *
gtracer_proxy_log_substance(GTracerProxyLog * proxy)
{
  g_return_val_if_fail (proxy, NULL);
  return proxy->substance;
}

/* arch-tag: f3d48224-b44d-4c2d-9275-5cdfe51e781e
   (do not change this comment) */
