/* queuek.c --- queue of logs

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "plugin.h"

typedef struct _GTracerLogQueue GTracerLogQueue;
struct _GTracerLogQueue
{
  GAsyncQueue*       log_queue;
  guint              block_count;
  GThread*           block_thread;
  GSList*            autounref_pool;
};
static GTracerLogQueue gtracer_queue;

static gboolean gtracer_queue_is_locked    (void);
static void     gtracer_autounref_pool_add (GTracerLog * log);
static void     gtracer_autounref_pool_run (void);

void
gtracer_queue_init    (void)
{
  gtracer_queue.log_queue      = g_async_queue_new();
  gtracer_queue.block_count    = 0;
  gtracer_queue.block_thread   = NULL;
  gtracer_queue.autounref_pool = NULL;
}

static gboolean
gtracer_queue_is_locked (void)
{
  if (gtracer_queue.block_thread == g_thread_self())
    return (gtracer_queue.block_count == 0)? FALSE: TRUE;
  else
    return FALSE;
}

void
gtracer_queue_push_log (GTracerLog * log)
{
  g_return_if_fail (log);
  if (!gtracer_queue_is_locked())
    {
      gtracer_log_ref(log);
      g_async_queue_push (gtracer_queue.log_queue, log);
    }
}

GTracerLog*
gtracer_queue_pop_log (void)
{
  GTracerLog * log = g_async_queue_try_pop (gtracer_queue.log_queue);
  if (log)
    gtracer_autounref_pool_add(log);
  return log;
}

void
gtracer_queue_freeze (void)
{
  GThread * thread = g_thread_self();
  g_return_if_fail ((gtracer_queue.block_thread == NULL)
		    || (gtracer_queue.block_thread 
			&& gtracer_queue.block_thread == thread));
  gtracer_queue.block_thread = thread;
  gtracer_queue.block_count++;
  
}

void
gtracer_queue_thaw    (void)
{
  g_return_if_fail (gtracer_queue.block_thread == g_thread_self());
  gtracer_queue.block_count--;
  if (gtracer_queue.block_count == 0)
    {
      gtracer_autounref_pool_run();
      gtracer_queue.block_thread = NULL;
    }
}


/* -------------------------------------------------------------------
   Autounref pool operations
   ----------------------------------------------------------------- */ 
static void
gtracer_autounref_pool_add(GTracerLog * log)
{
  gtracer_queue.autounref_pool = g_slist_prepend(gtracer_queue.autounref_pool, 
						 log);
 }
 
static void
gtracer_autounref_pool_run(void)
{
  GSList * autounref_pool = gtracer_queue.autounref_pool;
  GTracerLog * log;
  while (autounref_pool)
    {
      log = autounref_pool->data;
      gtracer_log_unref(log);
      autounref_pool = g_slist_next(autounref_pool);
    }
  g_slist_free(gtracer_queue.autounref_pool);
  gtracer_queue.autounref_pool = NULL;
}

/* arch-tag: 9c03ac99-ea9a-4f52-8c74-94e601adfffe
   (do not change this comment) */
