/* service.h --- decls of services common routines

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

/* gtracer <- service, 
   service <-> service */

#ifndef __GTRACER_SERVICE_H__
#define __GTRACER_SERVICE_H__

#include "plugin.h"
#include <glib.h>
#include <addrslv.h>

G_BEGIN_DECLS

void gtracer_nil_service_init     (GError ** err);
void gtracer_proxy_service_init   (GError ** err);
void gtracer_pmaps_service_init   (GError ** err);
void gtracer_signal_service_init  (GError ** err);
void gtracer_environ_service_init (GError ** err);
void gtracer_fd_service_init      (GError ** err);
void gtracer_usignal_service_init (GError ** err);
void gtracer_source_service_init  (GError ** err);

/* 
 * Nil log
 */
GTracerLog*   gtracer_nil_log        (void);
gboolean      gtracer_log_is_nil     (GTracerLogProtected * log);

/*
 * Proxy
 */
GTracerLog*   gtracer_proxy_log_new (GTracerLog * log);
gboolean      gtracer_log_is_proxy  (GTracerLogProtected * log);

/*
 * PMaps
 */

extern GType AddrslvSymbolInfoType;

AddrslvSymbolInfo * gtracer_pmaps_address_resolve    (guint address,
						      GError ** err);

gchar *             gtracer_pmaps_message_new        (AddrslvSymbolInfo * symbol_info,
						      guint address);

PMapsEntry        * gtracer_pmaps_address_find_entry (guint address,
						      GError ** err);

/* TRUE: Accept, FALSE: Reject */
gboolean            gtracer_pmaps_address_filter (guint address,
						  GError ** err);

/*
 * Backtrace
 */
void gtrace_backtrace_show_backtrace(const gchar * header, GSList * bt);

G_END_DECLS

#endif /* Not def: __GTRACER_SERVICE_H__ */

/* arch-tag: 48f90327-54e5-4747-80c5-cfb22582177e
   (do not change this comment) */
