/* signal_filter.c --- filter signal logs

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "signal_service.h"
#include "service.h"
#include "plugin.h"
#include "utils.h"

#include <ginspector/ginspector-tag.h>
#include <ginspector/ginspector-gtk-tag.h>

  /* Next one is defined in gtype-utils.h in G-Inspector */
extern GList * g_type_get_type_list (void);

#include <string.h>
#include <gset.h>

#include "i18n.h"


static gboolean gtracer_callback_filter_func   (GTracerLogProtected * log,
					        gpointer filter_data,
						GTracerProxyLog * proxy_log);
static gboolean gtracer_connect_filter_func    (GTracerLogProtected * log,
					        gpointer filter_data,
						GTracerProxyLog * proxy_log);
static gboolean gtracer_disconnect_filter_func (GTracerLogProtected * log,
						gpointer filter_data,
						GTracerProxyLog * proxy_log);
static gboolean gtracer_emit_filter_func       (GTracerLogProtected * log,
					        gpointer filter_data,
						GTracerProxyLog * proxy_log);

typedef struct _GTracerSignalFilter GTracerSignalFilter;
typedef struct _GTracerSignalNameFilter GTracerSignalNameFilter;

typedef struct _GTracerCallbackFilter GTracerCallbackFilter;
typedef struct _GTracerConnectFilter GTracerConnectFilter;
typedef struct _GTracerDisconnectFilter GTracerDisconnectFilter;
typedef struct _GTracerEmitFilter GTracerEmitFilter;

struct _GTracerSignalNameFilter
{
  GSet*         session;
  GSet*         static_filter;
  GHashTable*   frequency;
  GtkWidget*    window;
  GtkTreeModel* model;
};

struct _GTracerCallbackFilter
{
  gboolean   active;
  GtkWidget* active_button;
  gboolean   pmaps_sensitive;
  GtkWidget* pmaps_sensitive_button;
  GtkWidget* static_def_button;
};

struct _GTracerEmitFilter
{
  gboolean   active;
  GtkWidget* active_button;
  GtkWidget* static_def_button;
};

struct _GTracerConnectFilter
{
  gboolean   active;
  GtkWidget* active_button;
  gboolean   pmaps_sensitive;
  GtkWidget* pmaps_sensitive_button;
};

struct _GTracerDisconnectFilter
{
  gboolean   active;
  GtkWidget* active_button;
  gboolean   pmaps_sensitive;
  GtkWidget* pmaps_sensitive_button;
};

struct _GTracerSignalFilter
{
  GTracerSignalNameFilter name_filter;
  GTracerCallbackFilter   callback;
  GTracerConnectFilter    connect;
  GTracerDisconnectFilter disconnect;
  GTracerEmitFilter       emit;
};


static gchar * gtracer_signal_name_static_filter_initial_value[] = {
  "notify", "name-changed", "invalidate-preview", "dirty",
  "grab-notify", "parent-set", "show", "screen-changed",
  "hierarchy-changed", "add", "child-notify", "accel-closures-changed",
  "event", "event-after", "expose-event", "state-changed",
  "style-set", "size-request", "value-changed", "size-allocate",
  "configure-event", "changed", "realize", "map", "map-event",
  "enter-notify-event", "leave-notify-event", "motion-notify-event",
  /* END */
  NULL
};

static GTracerSignalFilter * gtracer_signal_filter_new                (void);

static void                  gtracer_signal_name_filter_init          (GTracerSignalNameFilter *  name_filter);
static GtkWidget*            gtracer_signal_name_filter_ui_new        (GTracerSignalNameFilter *  name_filter);
static void                  gtracer_signal_name_filter_ui_open       (GTracerSignalNameFilter *  name_filter);
static void                  gtracer_signal_name_filter_ui_update     (GtkButton * button, 
								       GTracerSignalNameFilter * name_filter);
static gboolean              gtracer_signal_name_filter_filter        (GTracerSignalNameFilter *  name_filter,
                                                                       const gchar *              signal_name);
static void                  gtracer_signal_name_filter_set_reject    (GTracerSignalNameFilter *  name_filter,
                                                                       gchar *                    signal_name,
                                                                       gboolean                   is_reject);
static void gtracer_signal_name_filter_opne_session                   (GTracerSignalNameFilter *  name_filter);
static void gtracer_signal_name_filter_close_session                  (GTracerSignalNameFilter *  name_filter);
static void gtracer_signal_name_filter_record_frequency               (GTracerSignalNameFilter *  name_filter,
								       const gchar *              signal_name);

static GtkWidget*            gtracer_signal_list_new                  (GTracerSignalNameFilter *  name_filter);
static void                  gtracer_signal_list_update               (GTracerSignalNameFilter *  name_filter);

static void                  gtracer_callback_filter_init             (GTracerCallbackFilter *    callback_filter);
static GtkWidget *           gtracer_callback_filter_ui_new           (GTracerCallbackFilter *    callback_filter,
                                                                       GTracerSignalNameFilter *  name_filter);

static void                  gtracer_connect_filter_init              (GTracerConnectFilter *     connect_filter);
static GtkWidget *           gtracer_connect_filter_ui_new            (GTracerConnectFilter *     connect_filter);

static void                  gtracer_disconnect_filter_init           (GTracerDisconnectFilter *  disconnect_filter);
static GtkWidget *           gtracer_disconnect_filter_ui_new         (GTracerDisconnectFilter *  disconnect_filter);

static void                  gtracer_emit_filter_init                 (GTracerEmitFilter *        emit_filter);
static GtkWidget *           gtracer_emit_filter_ui_new               (GTracerEmitFilter *        emit_filter,
                                                                       GTracerSignalNameFilter *  name_filter);

static void                  gtracer_signal_filter_hint               (GQuark                     log_type,
                                                                       gpointer                   filter_data,
                                                                       gboolean                   start);

/* ------------------------------------------------------------------
   Generic
   ------------------------------------------------------------------ */
void
gtracer_signal_filter_init (GError ** err)
{
  
  GTracerSignalFilter * filter;
  GTracerViewer * viewer;
  GtkWidget * ui;

  filter = gtracer_signal_filter_new();

  /* Callback */  
  viewer = gtracer_viewers_find (g_quark_from_string(GTRACER_CALLBACK_LOG_NAME));
  g_return_if_fail (viewer);
  gtracer_filters_install(GTRACER_CALLBACK_LOG_NAME,
			  gtracer_callback_filter_func,
			  gtracer_signal_filter_hint,
			  filter);
  ui = gtracer_callback_filter_ui_new(&filter->callback,
				      &filter->name_filter);
  gtracer_viewer_dock_filter_ui(viewer, ui);

  /* Connect */
  viewer = gtracer_viewers_find (g_quark_from_string(GTRACER_CONNECT_LOG_NAME));
  g_return_if_fail (viewer);
  gtracer_filters_install(GTRACER_CONNECT_LOG_NAME,
			  gtracer_connect_filter_func,
			  NULL,
			  filter);
  ui = gtracer_connect_filter_ui_new(&filter->connect);
  gtracer_viewer_dock_filter_ui(viewer, ui);

  /* Disconnect */
  viewer = gtracer_viewers_find (g_quark_from_string(GTRACER_DISCONNECT_LOG_NAME));
  g_return_if_fail (viewer);
  gtracer_filters_install(GTRACER_DISCONNECT_LOG_NAME,
			  gtracer_disconnect_filter_func,
			  NULL,
			  filter);
  ui = gtracer_disconnect_filter_ui_new(&filter->disconnect);
  gtracer_viewer_dock_filter_ui(viewer, ui);

  /* Emit */
  viewer = gtracer_viewers_find (g_quark_from_string(GTRACER_EMIT_LOG_NAME));
  g_return_if_fail (viewer);
  gtracer_filters_install(GTRACER_EMIT_LOG_NAME,
			  gtracer_emit_filter_func,
			  NULL,
			  filter);
  ui = gtracer_emit_filter_ui_new(&filter->emit,
				  &filter->name_filter);
  gtracer_viewer_dock_filter_ui(viewer, ui);
}

static GTracerSignalFilter *
gtracer_signal_filter_new(void)
{
  GTracerSignalFilter * filter = g_new(GTracerSignalFilter, 1);
  
  gtracer_signal_name_filter_init(&filter->name_filter);

  gtracer_callback_filter_init(&filter->callback);
  gtracer_connect_filter_init (&filter->connect);
  gtracer_disconnect_filter_init(&filter->disconnect);
  gtracer_emit_filter_init    (&filter->emit);

  return filter;
}

static void
gtracer_signal_filter_hint       (GQuark log_type,
				  gpointer filter_data,
				  gboolean start)
{
  GTracerSignalFilter * filter = filter_data;
  
  if (start == TRUE)
    gtracer_signal_name_filter_opne_session(&filter->name_filter);
  else
    gtracer_signal_name_filter_close_session(&filter->name_filter);
}

static void
gtracer_signal_name_filter_init       (GTracerSignalNameFilter * name_filter)
{
  gchar ** fq_name;
  
  name_filter->session 	     = NULL;

  name_filter->static_filter = g_set_new((GCompareFunc)strcmp, 
					 (GSetCopyFunc)g_strdup, 
					 g_free);
  for(fq_name = gtracer_signal_name_static_filter_initial_value; 
      *fq_name; 
      fq_name++)
    g_set_add(name_filter->static_filter, *fq_name);

  name_filter->frequency = g_hash_table_new(g_str_hash, g_str_equal);
  name_filter->window =    gtracer_signal_name_filter_ui_new(name_filter);
}

static gboolean
gtracer_signal_name_filter_filter     (GTracerSignalNameFilter * name_filter, 
				       const gchar * signal_name)
{
  gboolean reject;

  
  g_return_val_if_fail (name_filter, FALSE);

  if (name_filter->session 
      && (g_set_contains(name_filter->session, signal_name) == TRUE))
    return TRUE;
  
  reject = g_set_contains(name_filter->static_filter, signal_name);
  if (reject == TRUE) 
    return FALSE;

  if (name_filter->session)
    g_set_add(name_filter->session, signal_name);
  
  return TRUE;
}

static void
gtracer_signal_name_filter_opne_session (GTracerSignalNameFilter * name_filter)
{
  g_assert(name_filter->session == NULL);
  name_filter->session = g_set_new((GCompareFunc)strcmp, 
				   (GSetCopyFunc)g_strdup, 
				   g_free);
}

static void
gtracer_signal_name_filter_close_session (GTracerSignalNameFilter * name_filter)
{
  g_set_destroy(name_filter->session);
  name_filter->session = NULL;
}

static void
gtracer_signal_name_filter_set_reject (GTracerSignalNameFilter * name_filter, 
				       gchar * signal_name,
				       gboolean is_reject)
{
  g_return_if_fail (name_filter);
  g_return_if_fail (signal_name);

  if (is_reject) 
    {
      if (g_set_contains(name_filter->static_filter, signal_name) == FALSE)
	g_set_add(name_filter->static_filter, signal_name);
    } 
  else
    {
      if (g_set_contains(name_filter->static_filter, signal_name) == TRUE)
	g_set_remove(name_filter->static_filter, signal_name);
    }
}

static GtkWidget*
gtracer_signal_name_filter_ui_new(GTracerSignalNameFilter * name_filter)
{
  GtkWidget     * dialog;
  GtkWidget     * vbox;
  GtkWidget     * scrolled_window;
  GtkAdjustment * adjustment;  
  GtkWidget     * list;  
  GtkWidget     * update_button;

  dialog = gtk_dialog_new();
  g_inspector_tag_to_widget(dialog);
  gtk_window_set_title(GTK_WINDOW(dialog), _("Static Name Filter"));
  
#if 0
  update_button = gtk_button_new_with_label(_("Update List"));
  g_signal_connect(G_OBJECT(update_button),
		   "clicked",
		   G_CALLBACK(gtracer_signal_name_filter_ui_update),
		   name_filter);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     update_button,
		     FALSE,
		     FALSE,
		     2);
#endif /* 0 */
  
  gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE,
			GTK_RESPONSE_ACCEPT);
  

  
  vbox = GTK_DIALOG(dialog)->vbox;
  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  adjustment 	  = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrolled_window));
  g_inspector_tag_to_object(G_OBJECT(adjustment));
  adjustment 	  = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolled_window));
  g_inspector_tag_to_object(G_OBJECT(adjustment));
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				 GTK_POLICY_AUTOMATIC, 
				 GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 2);

  list = gtracer_signal_list_new(name_filter);
  gtk_widget_set_size_request (list, 240, 360);
  gtk_container_add(GTK_CONTAINER(scrolled_window),
		    list);

  return dialog;
}

static void
gtracer_signal_name_filter_ui_open(GTracerSignalNameFilter * name_filter)
{
  gtracer_signal_list_update(name_filter);
  gtk_widget_show_all(name_filter->window);
  gtk_dialog_run(GTK_DIALOG(name_filter->window));
  gtk_widget_hide(name_filter->window);
}

static void
gtracer_signal_name_filter_ui_update     (GtkButton * button, 
					  GTracerSignalNameFilter * name_filter)
{
  gtracer_signal_list_update(name_filter);
}

static void
gtracer_signal_name_filter_record_frequency (GTracerSignalNameFilter *  name_filter,
					     const gchar * signal_name)
{
  gpointer tmp;  
  tmp = g_hash_table_lookup(name_filter->frequency, signal_name);
  if (tmp) 
    {
      guint fq;
      fq       = GPOINTER_TO_UINT(tmp);
      fq = (fq == G_MAXUINT)? G_MAXUINT: fq + 1;
      g_hash_table_insert(name_filter->frequency,
			  signal_name,
			  GUINT_TO_POINTER(fq));
    }
  else
    g_hash_table_insert(name_filter->frequency,
			g_strdup(signal_name),
			GUINT_TO_POINTER(1));

}

/* ------------------------------------------------------------------
   Signal List
   ------------------------------------------------------------------ */
enum
{
  COL_SIGNAL,
  NUM_COLS
} ;

enum
{
  SORTID_CHECKED,
  SORTID_SIGNAL,
  SORTID_FREQUENCY,
};
static void gtracer_signal_list_toggled                (GtkCellRendererToggle *cell,
                                                        gchar                 *path_string,
                                                        gpointer               user_data);
static void gtracer_signal_list_extract_ignorance      (GtkTreeViewColumn     *col,
                                                        GtkCellRenderer       *renderer,
                                                        GtkTreeModel          *model,
                                                        GtkTreeIter           *iter,
                                                        gpointer               user_data);
static void gtracer_signal_list_extract_frequency      (GtkTreeViewColumn     *col,
                                                        GtkCellRenderer       *renderer,
                                                        GtkTreeModel          *model,
                                                        GtkTreeIter           *iter,
                                                        gpointer               user_data);
static void gtracer_signal_list_extract_signal         (GtkTreeViewColumn     *col,
                                                        GtkCellRenderer       *renderer,
                                                        GtkTreeModel          *model,
                                                        GtkTreeIter           *iter,
                                                        gpointer               user_data);
static gint gtracer_signal_list_checked_compare_func    (GtkTreeModel          *model,
                                                         GtkTreeIter           *a,
							 GtkTreeIter           *b,
							 gpointer               user_data);
static gint gtracer_signal_list_frequency_compare_func (GtkTreeModel          *model,
                                                        GtkTreeIter           *a,
                                                        GtkTreeIter           *b,
                                                        gpointer               user_data);
static gint gtracer_signal_list_name_compare_func      (GtkTreeModel          *model,
                                                        GtkTreeIter           *a,
                                                        GtkTreeIter           *b,
                                                        gpointer               user_data);

static GtkWidget *
gtracer_signal_list_new(GTracerSignalNameFilter *  name_filter)
{
  GtkWidget         *view;
  GtkListStore      *store;
  GtkCellRenderer   *toggle_renderer;
  GtkCellRenderer   *text_renderer;
  gint               position;
  GtkTreeViewColumn *col;
  GtkTreeSelection  *sel;  
  GtkTreeSortable * sortable;

  
  view = gtk_tree_view_new();
  
  toggle_renderer = gtk_cell_renderer_toggle_new();
  g_inspector_tag_to_object(G_OBJECT(toggle_renderer));
  g_signal_connect(toggle_renderer, "toggled", 
		   G_CALLBACK(gtracer_signal_list_toggled), name_filter);
  g_object_set(toggle_renderer, "activatable", TRUE, NULL);
  position = 0;
  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(view),
							position,
							_("Ignore"),
							toggle_renderer,
							gtracer_signal_list_extract_ignorance,
							name_filter,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(view), position - 1);
  gtk_tree_view_column_set_sort_column_id(col, SORTID_CHECKED);
  gtracer_register_tree_view_column(col);

  text_renderer = gtk_cell_renderer_text_new ();
  g_inspector_tag_to_object(G_OBJECT(text_renderer));
  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(view),
							position,
							_("Frequency"),
							text_renderer,
							gtracer_signal_list_extract_frequency,
							name_filter,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(view), position - 1);
  gtk_tree_view_column_set_sort_column_id(col, SORTID_FREQUENCY);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(view),
							position,
							_("Signal"),
							text_renderer,
							gtracer_signal_list_extract_signal,
							name_filter,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(view), position - 1);
  gtk_tree_view_column_set_sort_column_id(col, SORTID_SIGNAL);
  gtracer_register_tree_view_column(col);

  store = gtk_list_store_new (NUM_COLS, G_TYPE_STRING);
  g_inspector_tag_to_object(G_OBJECT(store));
  gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL(store));
  name_filter->model = GTK_TREE_MODEL(store);
  sortable = GTK_TREE_SORTABLE(store);
  gtk_tree_sortable_set_sort_func(sortable, SORTID_CHECKED,
				  gtracer_signal_list_checked_compare_func,
				  name_filter, NULL);
  gtk_tree_sortable_set_sort_func(sortable, SORTID_FREQUENCY,
				  gtracer_signal_list_frequency_compare_func,
				  name_filter, NULL);
  
  gtk_tree_sortable_set_sort_func(sortable, SORTID_SIGNAL,
				  gtracer_signal_list_name_compare_func,
				  name_filter, NULL);
  /* g_object_unref (model); */
  
  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
  g_inspector_tag_to_object(G_OBJECT(sel));
  return view;
}

static void
gtracer_signal_list_toggled           (GtkCellRendererToggle *cell,
				       gchar                 *path_string,
				       gpointer               user_data)
{
  GTracerSignalNameFilter *name_filter;
  GtkTreeIter              iter;
  gchar                   *signal_name;
  gboolean                 memberp;
  
  name_filter = user_data;
  gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(name_filter->model), 
				      &iter, 
				      path_string);
  
  gtk_tree_model_get(GTK_TREE_MODEL(name_filter->model), 
		     &iter, 
		     COL_SIGNAL, 
		     &signal_name, 
		     -1);

  memberp = g_set_contains(name_filter->static_filter, signal_name);
  gtracer_signal_name_filter_set_reject(name_filter,
					signal_name,
					!memberp);
  g_free(signal_name);
}
  
static void
gtracer_signal_list_extract_ignorance (GtkTreeViewColumn     *col,
				       GtkCellRenderer       *renderer,
				       GtkTreeModel          *model,
				       GtkTreeIter           *iter,
				       gpointer               user_data)
{
  GTracerSignalNameFilter *name_filter;
  gchar                   *signal_name;
  gboolean                 memberp;
  
  name_filter = user_data;
  gtk_tree_model_get (model, iter, COL_SIGNAL, &signal_name, -1);
  memberp = g_set_contains (name_filter->static_filter, signal_name);
  g_object_set (renderer, "active", memberp, NULL); 
  g_free(signal_name);
}

static void
gtracer_signal_list_extract_frequency (GtkTreeViewColumn     *col,
				       GtkCellRenderer       *renderer,
				       GtkTreeModel          *model,
				       GtkTreeIter           *iter,
				       gpointer               user_data)
{
  GTracerSignalNameFilter *name_filter;
  gchar                   *signal_name;
  gpointer tmp;
  gchar   *fq_str;
  
  name_filter = user_data;
  gtk_tree_model_get (model, iter, COL_SIGNAL, &signal_name, -1);
  tmp 	 = g_hash_table_lookup(name_filter->frequency, signal_name);
  fq_str = g_strdup_printf("%u", tmp? GPOINTER_TO_UINT(tmp): 0);
  g_object_set(G_OBJECT(renderer), "text", fq_str, NULL);
  g_free(fq_str);
  g_free(signal_name);
}

static void
gtracer_signal_list_extract_signal    (GtkTreeViewColumn     *col,
				       GtkCellRenderer       *renderer,
				       GtkTreeModel          *model,
				       GtkTreeIter           *iter,
				       gpointer               user_data)
{
  GTracerSignalNameFilter *name_filter;
  gchar                   *signal_name;

  name_filter = user_data;
  gtk_tree_model_get (model, iter, COL_SIGNAL, &signal_name, -1);
  g_object_set(G_OBJECT(renderer), "text", signal_name, NULL);
  g_free(signal_name);
}

static void
gtracer_signal_list_update               (GTracerSignalNameFilter *  name_filter)
{
  GtkTreeIter iter;
  GList *type_list;
  GList *tmp;
  GType  type;
  GSet  *installed_signals;
  
  gtk_list_store_clear (GTK_LIST_STORE(name_filter->model));
  installed_signals = g_set_new((GCompareFunc)strcmp, 
				(GSetCopyFunc)g_strdup, 
				g_free);
  
  type_list = g_type_get_type_list ();

  tmp = type_list;
  while (tmp != NULL) 
    {
      guint *signal_ids;
      guint  n_signal_ids;
      gint   i;
      GSignalQuery signal_query;
      
      type = (GType)tmp->data;
      tmp = g_list_next (tmp);

      if (!(G_TYPE_IS_INSTANTIATABLE (type) ||
	    G_TYPE_IS_INTERFACE (type)))
	continue;
      
      signal_ids = g_signal_list_ids (type, &n_signal_ids);
      for (i = 0; i < n_signal_ids; ++i)
	{
	  g_signal_query (signal_ids[i], &signal_query);
	  
	  if (g_set_contains(installed_signals, signal_query.signal_name) == TRUE)
	    continue ;
	  
	  gtk_list_store_append (GTK_LIST_STORE(name_filter->model), &iter);
	  gtk_list_store_set (GTK_LIST_STORE(name_filter->model), 
			      &iter, COL_SIGNAL, signal_query.signal_name, -1);
	  g_set_add(installed_signals, signal_query.signal_name);
	}
      if (n_signal_ids > 0)
	g_free (signal_ids);
    }
  g_list_free (type_list);
  g_set_destroy(installed_signals);
}

static gint
gtracer_signal_list_checked_compare_func    (GtkTreeModel          *model,
					     GtkTreeIter           *a,
					     GtkTreeIter           *b,
					     gpointer               user_data)
{
  gchar * a_signal;
  gchar * b_signal;
  gboolean a_checked;
  gboolean b_checked;
  GTracerSignalNameFilter *name_filter;
  
  gtk_tree_model_get(model, a, COL_SIGNAL, &a_signal, -1);
  gtk_tree_model_get(model, b, COL_SIGNAL, &b_signal, -1);

  name_filter = user_data;
  a_checked   = g_set_contains (name_filter->static_filter, a_signal);
  b_checked   = g_set_contains (name_filter->static_filter, b_signal);

  g_free(b_signal);
  g_free(a_signal);
  
  if (a_checked == b_checked) 
    return 0;
  else if (a_checked < b_checked) 
    return 1;
  else
    return -1;
}

static gint
gtracer_signal_list_frequency_compare_func (GtkTreeModel          *model,
					    GtkTreeIter           *a,
					    GtkTreeIter           *b,
					    gpointer               user_data)
{
  gchar * a_signal;
  gchar * b_signal;
  gpointer tmp;
  guint a_fq;
  guint b_fq;
  gint comparison;
  GTracerSignalNameFilter *name_filter;

  gtk_tree_model_get(model, a, COL_SIGNAL, &a_signal, -1);
  gtk_tree_model_get(model, b, COL_SIGNAL, &b_signal, -1);

  name_filter = user_data;
  tmp 	      = g_hash_table_lookup(name_filter->frequency, a_signal);
  a_fq 	      = GPOINTER_TO_UINT(tmp);
  tmp 	      = g_hash_table_lookup(name_filter->frequency, b_signal);
  b_fq 	      = GPOINTER_TO_UINT(tmp);
  
  if (a_fq < b_fq) 
    comparison = 1;
  else if (a_fq == b_fq)
    comparison = 0;
  else 
    comparison = -1;
  
  g_free(b_signal);
  g_free(a_signal);
  
  return comparison;
}

static gint
gtracer_signal_list_name_compare_func      (GtkTreeModel          *model,
					    GtkTreeIter           *a,
					    GtkTreeIter           *b,
					    gpointer               user_data)
{
  gchar * a_signal;
  gchar * b_signal;
  gint comparison;
  
  gtk_tree_model_get(model, a, COL_SIGNAL, &a_signal, -1);
  gtk_tree_model_get(model, b, COL_SIGNAL, &b_signal, -1);

  comparison = strcmp(a_signal, b_signal);
  
  g_free(b_signal);
  g_free(a_signal);
  
  return comparison;
}


/* ------------------------------------------------------------------
   Callback
   ------------------------------------------------------------------ */
static void gtracer_callback_filter_active_toggled(GtkToggleButton* toggle_button,
						     gpointer user_data);
static void gtracer_callback_filter_pmaps_sensitive_toggled(GtkToggleButton* toggle_button,
							    gpointer user_data);

static void gtracer_callback_filter_create_static_def_window(GtkButton * button,
							     gpointer user_data);

static gboolean
gtracer_callback_filter_func(GTracerLogProtected * log,
			     gpointer filter_data,
			     GTracerProxyLog * proxy_log)
{
  GTracerCallbackLog * callback_log = (GTracerCallbackLog *)log;
  const gchar * signal_name;
  GTracerSignalFilter * filter = filter_data;
  GTracerCallbackFilter * callback_filter = &filter->callback;
  gpointer callback_log_handler;
  GError * err = NULL;
  
  if  (callback_filter->active == FALSE)
    return FALSE;

  signal_name = gtracer_signal_log_signal((GTracerSignalLog *)log);
  if (gtracer_signal_name_filter_filter(&filter->name_filter,
					signal_name) == FALSE )
    return FALSE;
  
  /* pmaps filter */
  callback_log_handler = gtracer_callback_log_handler(callback_log);
  if ((callback_filter->pmaps_sensitive)
      && (FALSE == gtracer_pmaps_address_filter(GPOINTER_TO_UINT(callback_log_handler),
						&err)))
    {
      /* TODO: Catch an error.*/
      if (err) 
	g_clear_error(&err);
      return FALSE;
    }

  return TRUE;
}

static void
gtracer_callback_filter_init(GTracerCallbackFilter * callback_filter)
{
  callback_filter->active 	   	 = TRUE;
  callback_filter->pmaps_sensitive 	 = TRUE;
}

static GtkWidget *
gtracer_callback_filter_ui_new(GTracerCallbackFilter * callback_filter,
			       GTracerSignalNameFilter *  name_filter)
{
  GtkWidget * frame;
  GtkWidget * vbox;
  GtkWidget * check_button;
  GtkWidget * static_def_button;
  
  g_return_val_if_fail (callback_filter, NULL);
  
  frame = gtk_frame_new(NULL);
  g_inspector_tag_to_widget (frame);
  
  check_button = gtk_check_button_new_with_label("Callback");
  gtk_frame_set_label_widget(GTK_FRAME(frame), check_button);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
			       callback_filter->active);
  g_signal_connect(G_OBJECT(check_button),
		   "toggled",
		   G_CALLBACK(gtracer_callback_filter_active_toggled),
		   callback_filter);
  callback_filter->active_button = check_button;

  
  vbox = gtk_vbox_new(FALSE, 4);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
  gtk_container_add(GTK_CONTAINER(frame), vbox);

  check_button = gtk_check_button_new_with_label(_("PMaps Sensitive"));
  gtk_box_pack_start(GTK_BOX(vbox), check_button,
		     FALSE, FALSE, 2);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
			       callback_filter->pmaps_sensitive);
  g_signal_connect(G_OBJECT(check_button),
		   "toggled",
		   G_CALLBACK(gtracer_callback_filter_pmaps_sensitive_toggled),
		   callback_filter);
  callback_filter->pmaps_sensitive_button = check_button;
  
  static_def_button = gtk_button_new_with_label(_("Tune Static Name Filter..."));
  gtk_box_pack_start(GTK_BOX(vbox), static_def_button,
		     FALSE, FALSE, 2);
  gtk_widget_set_sensitive(static_def_button,
			   callback_filter->active);
  g_signal_connect(G_OBJECT(static_def_button),
		   "clicked",
		   G_CALLBACK(gtracer_callback_filter_create_static_def_window),
		   name_filter);
  callback_filter->static_def_button = static_def_button;

  return frame;
}

static void
gtracer_callback_filter_active_toggled(GtkToggleButton* toggle_button,
					 gpointer user_data)
{
  GTracerCallbackFilter * callback_filter = user_data;
  
  callback_filter->active = gtk_toggle_button_get_active(toggle_button);
  gtk_widget_set_sensitive(callback_filter->pmaps_sensitive_button,
			   callback_filter->active);
  gtk_widget_set_sensitive(callback_filter->static_def_button,
			   callback_filter->active);
}

static void
gtracer_callback_filter_pmaps_sensitive_toggled(GtkToggleButton* toggle_button,
						gpointer user_data)
{
  GTracerCallbackFilter * callback_filter = user_data;
  callback_filter->pmaps_sensitive = gtk_toggle_button_get_active(toggle_button);
}

static void
gtracer_callback_filter_create_static_def_window(GtkButton * button,
						 gpointer user_data)
{
  gtracer_signal_name_filter_ui_open(user_data);
}


/* ------------------------------------------------------------------
   Connect
   ------------------------------------------------------------------ */
static void gtracer_connect_filter_active_toggled(GtkToggleButton* toggle_button, 
						  gpointer user_data);

static void gtracer_connect_filter_pmaps_senstive_toggled(GtkToggleButton* toggle_button,
							  gpointer user_data);

static gboolean
gtracer_connect_filter_func  (GTracerLogProtected * log,
			      gpointer filter_data,
			      GTracerProxyLog * proxy_log)
{
  GTracerSignalLog * signal_log = (GTracerSignalLog *)log;
  GTracerConnectLog * connect_log = (GTracerConnectLog *)log;

  GTracerSignalFilter * filter = filter_data;
  GTracerConnectFilter * connect_filter = &filter->connect;

  gpointer instance;  
  const gchar * signal_name;
  gpointer handler;
  GError * err = NULL;;
  
  signal_name = gtracer_signal_log_signal(signal_log);
  instance    = gtracer_signal_log_instance(signal_log);

  if (g_inspector_tag_check_object (instance))
    return FALSE;
  
  if (GTK_IS_WIDGET(instance)
      && g_inspector_tag_check_widget(instance))
    return FALSE;
  
  gtracer_connect_log_install_stock((GTracerConnectLog *)log);
  
  if (connect_filter->active == FALSE)
    return FALSE;


  handler = gtracer_connect_log_handler(connect_log);
  if ((connect_filter->pmaps_sensitive == TRUE)
      && (FALSE == gtracer_pmaps_address_filter(GPOINTER_TO_UINT(handler),
						&err)))
    {
      if (err) 
	g_clear_error(&err);
      return FALSE;
    }
  
  return TRUE;
}

static void
gtracer_connect_filter_init(GTracerConnectFilter * connect_filter)
{
  connect_filter->active 	  = FALSE;
  connect_filter->pmaps_sensitive = TRUE;
}

static GtkWidget *
gtracer_connect_filter_ui_new(GTracerConnectFilter * connect_filter)
{
  GtkWidget * frame;
  GtkWidget * vbox;
  GtkWidget * check_button;

  g_return_val_if_fail (connect_filter, NULL);
  
  frame = gtk_frame_new(NULL);
  g_inspector_tag_to_widget (frame);
  
  check_button = gtk_check_button_new_with_label("Connect");
  gtk_frame_set_label_widget(GTK_FRAME(frame), check_button);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
			       connect_filter->active);
  g_signal_connect(G_OBJECT(check_button),
		   "toggled",
		   G_CALLBACK(gtracer_connect_filter_active_toggled),
		   connect_filter);
  connect_filter->active_button = check_button;

  vbox = gtk_vbox_new(FALSE, 4);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
  gtk_container_add(GTK_CONTAINER(frame), vbox);
  
  check_button = gtk_check_button_new_with_label(_("PMaps Sensitive"));
  gtk_box_pack_start(GTK_BOX(vbox), check_button,
		     FALSE, FALSE, 2);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
			       connect_filter->pmaps_sensitive);
  gtk_widget_set_sensitive(check_button,
			   connect_filter->active);
  g_signal_connect(G_OBJECT(check_button),
		   "toggled",
		   G_CALLBACK(gtracer_connect_filter_pmaps_senstive_toggled),
		   connect_filter);
  connect_filter->pmaps_sensitive_button = check_button;
  
  
  return frame;
}

static void
gtracer_connect_filter_active_toggled(GtkToggleButton* toggle_button,
				      gpointer user_data)
{
  GTracerConnectFilter * connect_filter = user_data;
  
  connect_filter->active = gtk_toggle_button_get_active(toggle_button);
  gtk_widget_set_sensitive(connect_filter->pmaps_sensitive_button,
			   connect_filter->active);
}

static void
gtracer_connect_filter_pmaps_senstive_toggled(GtkToggleButton* toggle_button,
					      gpointer user_data)
{
  GTracerConnectFilter * connect_filter = user_data;
  
  connect_filter->pmaps_sensitive = gtk_toggle_button_get_active(toggle_button);
}


/* ------------------------------------------------------------------
   Disconnect
   ------------------------------------------------------------------ */
static void gtracer_disconnect_filter_active_toggled(GtkToggleButton* toggle_button, 
						  gpointer user_data);

static void gtracer_disconnect_filter_pmaps_senstive_toggled(GtkToggleButton* toggle_button,
							     gpointer user_data);

static gboolean
gtracer_disconnect_filter_func  (GTracerLogProtected * log,
				 gpointer filter_data,
				 GTracerProxyLog * proxy_log)
{
  GTracerSignalLog * signal_log = (GTracerSignalLog *)log;
  GTracerDisconnectLog * disconnect_log = (GTracerDisconnectLog *)log;

  GTracerSignalFilter * filter = filter_data;
  GTracerDisconnectFilter * disconnect_filter = &filter->disconnect;

  gpointer instance;
  gpointer handler;
  GError * err = NULL;;
  
  instance    = gtracer_signal_log_instance(signal_log);

  if (g_inspector_tag_check_object (instance))
    return FALSE;
  
  if (GTK_IS_WIDGET(instance)
      && g_inspector_tag_check_widget(instance))
    return FALSE;
  
  if (disconnect_filter->active == FALSE)
    return FALSE;

  handler = gtracer_disconnect_log_handler(disconnect_log);
  if (handler &&
      (disconnect_filter->pmaps_sensitive == TRUE)
      && (FALSE == gtracer_pmaps_address_filter(GPOINTER_TO_UINT(handler),
						&err)))
    {
      if (err) 
	g_clear_error(&err);
      return FALSE;
    }
  
  return TRUE;
}

static void
gtracer_disconnect_filter_init(GTracerDisconnectFilter * disconnect_filter)
{
  disconnect_filter->active = FALSE;
  disconnect_filter->pmaps_sensitive = TRUE;
}

static GtkWidget *
gtracer_disconnect_filter_ui_new(GTracerDisconnectFilter * disconnect_filter)
{
  GtkWidget * frame;
  GtkWidget * vbox;
  GtkWidget * check_button;

  g_return_val_if_fail (disconnect_filter, NULL);
  
  frame = gtk_frame_new(NULL);
  g_inspector_tag_to_widget (frame);

  check_button = gtk_check_button_new_with_label("Disconnect");
  gtk_frame_set_label_widget(GTK_FRAME(frame), check_button);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
			       disconnect_filter->active);
  g_signal_connect(G_OBJECT(check_button),
		   "toggled",
		   G_CALLBACK(gtracer_disconnect_filter_active_toggled),
		   disconnect_filter);
  disconnect_filter->active_button = check_button;
  
  vbox = gtk_vbox_new(FALSE, 4);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
  gtk_container_add(GTK_CONTAINER(frame), vbox);

  check_button = gtk_check_button_new_with_label(_("PMaps Sensitive"));
  gtk_box_pack_start(GTK_BOX(vbox), check_button,
		     FALSE, FALSE, 2);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
			       disconnect_filter->pmaps_sensitive);
  gtk_widget_set_sensitive(check_button,
			   disconnect_filter->active);
  g_signal_connect(G_OBJECT(check_button),
		   "toggled",
		   G_CALLBACK(gtracer_disconnect_filter_pmaps_senstive_toggled),
		   disconnect_filter);
  disconnect_filter->pmaps_sensitive_button = check_button;

  return frame;
}

static void
gtracer_disconnect_filter_active_toggled(GtkToggleButton* toggle_button,
				      gpointer user_data)
{
  GTracerDisconnectFilter * disconnect_filter = user_data;
  
  disconnect_filter->active = gtk_toggle_button_get_active(toggle_button);
  gtk_widget_set_sensitive(disconnect_filter->pmaps_sensitive_button,
			   disconnect_filter->active);
}

static void
gtracer_disconnect_filter_pmaps_senstive_toggled(GtkToggleButton* toggle_button,
						 gpointer user_data)
{
  GTracerDisconnectFilter * disconnect_filter = user_data;
  
  disconnect_filter->pmaps_sensitive = gtk_toggle_button_get_active(toggle_button);
}


/* ------------------------------------------------------------------
   Emit
   ------------------------------------------------------------------ */
static void gtracer_emit_filter_active_toggled           (GtkToggleButton*  toggle_button,
                                                          gpointer          user_data);
static void gtracer_emit_filter_create_static_def_window (GtkButton *       button,
                                                          gpointer          user_data);

static gboolean
gtracer_emit_filter_func(GTracerLogProtected * log,
			 gpointer filter_data,
			 GTracerProxyLog * proxy_log)
{
  GTracerSignalLog * signal_log = (GTracerSignalLog *)log;
  const gchar * signal_name;
  GTracerSignalFilter * filter = filter_data;
  GTracerEmitFilter * emit_filter = &filter->emit;
  gpointer instance;

  g_return_val_if_fail (signal_log, FALSE);
  
  signal_name = gtracer_signal_log_signal(signal_log);
  instance    = gtracer_signal_log_instance(signal_log);

  if (strcmp(signal_name, "destroy") == 0)
    gtracer_make_object_zombie(instance);

  if (g_inspector_tag_check_object (instance))
    return FALSE;
  
  if (GTK_IS_WIDGET(instance)
      && g_inspector_tag_check_widget(instance))
    return FALSE;
  

  gtracer_signal_name_filter_record_frequency(&filter->name_filter,
					      signal_name);
  
  if (emit_filter->active == FALSE)
    return FALSE;

  
  return gtracer_signal_name_filter_filter(&filter->name_filter,
					   signal_name);
}

static void
gtracer_emit_filter_init(GTracerEmitFilter * emit_filter)
{
  emit_filter->active 	   	     = FALSE;
}

static GtkWidget *
gtracer_emit_filter_ui_new(GTracerEmitFilter * emit_filter,
			   GTracerSignalNameFilter *  name_filter)
{
  GtkWidget * frame;
  GtkWidget * vbox;
  GtkWidget * check_button;
  GtkWidget * static_def_button;

  g_return_val_if_fail (emit_filter, NULL);
  
  frame = gtk_frame_new(NULL);
  g_inspector_tag_to_widget (frame);

  check_button = gtk_check_button_new_with_label("Emit");
  gtk_frame_set_label_widget(GTK_FRAME(frame), check_button);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
			       emit_filter->active);
  g_signal_connect(G_OBJECT(check_button),
		   "toggled",
		   G_CALLBACK(gtracer_emit_filter_active_toggled),
		   emit_filter);
  emit_filter->active_button = check_button;
  
  vbox = gtk_vbox_new(FALSE, 4);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
  gtk_container_add(GTK_CONTAINER(frame), vbox);

  static_def_button = gtk_button_new_with_label(_("Tune Static Name Filter..."));
  gtk_box_pack_start(GTK_BOX(vbox), static_def_button,
		     FALSE, FALSE, 2);
  gtk_widget_set_sensitive(static_def_button,
			   emit_filter->active);
  g_signal_connect(G_OBJECT(static_def_button),
		   "clicked",
		   G_CALLBACK(gtracer_emit_filter_create_static_def_window),
		   name_filter);
  emit_filter->static_def_button  = static_def_button;

  return frame;
}

static void
gtracer_emit_filter_active_toggled(GtkToggleButton* toggle_button,
				   gpointer user_data)
{
  GTracerEmitFilter * emit_filter = user_data;
  
  emit_filter->active = gtk_toggle_button_get_active(toggle_button);
  gtk_widget_set_sensitive(emit_filter->static_def_button,
			   emit_filter->active);
}

static void
gtracer_emit_filter_create_static_def_window(GtkButton * button,
					     gpointer user_data)
{
  gtracer_signal_name_filter_ui_open(user_data);
}

/* arch-tag: 593ef058-1d2f-462a-8b99-940da4092800
   (do not change this comment) */
