/* signal_service.c --- implementation of signal pricate routines

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "service.h"
#include "signal_service.h"

void
gtracer_signal_service_init (GError ** err)
{
  GError * tmp_error = NULL;
  gtracer_signal_log_class_init (&tmp_error);
  if (tmp_error) 
    {
      g_propagate_error(err, tmp_error);
      return ;
    }
  
  gtracer_signal_viewer_init(&tmp_error);
  if (tmp_error) 
    {
      g_propagate_error(err, tmp_error);
      return ;
    }
  
  gtracer_signal_filter_init (&tmp_error);
  if (tmp_error) 
    {
      g_propagate_error(err, tmp_error);
      return ;
    }
}

void
gtracer_signal_log_init       (GTracerSignalLog * log, 
			       GQuark log_type, 
			       GTracerLogProtectedOperations * lops,
			       gpointer instance,
			       GSList* backtrace, 
			       GTracerSignalLogOperations *slops)
{
  g_return_if_fail (log);
  g_return_if_fail (lops);
  g_return_if_fail (instance);
  /* g_return_if_fail (slops); */
  
  gtracer_log_init((GTracerLogProtected *)log, log_type, lops);
  log->instance  = g_object_ref(instance);
  log->backtrace = g_slist_reverse(backtrace);
  log->slops 	 = slops;
}

void
gtracer_signal_log_fin        (GTracerSignalLog * log)
{
  g_return_if_fail (log);
  g_return_if_fail (log->instance);

  gtracer_log_fin((GTracerLogProtected *)log);
  
  g_object_unref(log->instance);
  log->instance = NULL;
  g_slist_free(log->backtrace);
}

gpointer
gtracer_signal_log_instance   (GTracerSignalLog * log)
{
  g_return_val_if_fail (log, NULL);
  return log->instance;
}

const gchar*   
gtracer_signal_log_class_name (GTracerSignalLog * log)
{
  gpointer instance = gtracer_signal_log_instance(log);
  
  g_return_val_if_fail (instance, NULL);
  
  return G_OBJECT_TYPE_NAME(instance);
}

const gchar*
gtracer_signal_log_signal          (GTracerSignalLog * log)
{
  g_return_val_if_fail (log, NULL);
  if (log->slops == NULL)
    return NULL;
  
  if (log->slops->extract_signal == NULL) 
    return NULL;

  return log->slops->extract_signal(log);
}

GSList*
gtracer_signal_log_backtrace       (GTracerSignalLog * log)
{
  g_return_val_if_fail (log, NULL);
  return log->backtrace;
}

gchar*
gtracer_signal_log_create_notes(GTracerSignalLog * log)
{
  g_return_val_if_fail (log, NULL);

  if (log->slops == NULL)
    return NULL;
  
  if (log->slops->create_notes == NULL) 
    return NULL;

  return log->slops->create_notes (log);
}

gchar*
gtracer_signal_log_create_message (GTracerSignalLog * log)
{
  g_return_val_if_fail (log, NULL);

  if (log->slops == NULL)
    return NULL;
  
  if (log->slops->create_message == NULL) 
    return NULL;

  return log->slops->create_message (log);
}
/* arch-tag: 9c6d6783-f009-4c5a-bda4-fcce6b356d5c
   (do not change this comment) */
