/* signal_service.h --- decls of signal pricate routines

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#ifndef __GTRACER_SIGNAL_SERVICE_H__
#define __GTRACER_SIGNAL_SERVICE_H__

#include "plugin.h"
#include "gtracer.h"
#include <glib.h>

G_BEGIN_DECLS

void gtracer_signal_log_class_init (GError ** err);
void gtracer_signal_viewer_init    (GError ** err);
void gtracer_signal_filter_init    (GError ** err);

typedef struct _GTracerSignalLog GTracerSignalLog;
typedef struct _GTracerSignalLogOperations GTracerSignalLogOperations;
struct _GTracerSignalLog
{
  GTracerLogProtected log_protected;
  gpointer            instance;
  GSList*             backtrace;
  
  GTracerSignalLogOperations *slops;
};

struct _GTracerSignalLogOperations
{
  const gchar* (* extract_signal) (GTracerSignalLog * log); /* may return NULL */
  gchar*       (* create_notes)   (GTracerSignalLog * log);
  gchar*       (* create_message) (GTracerSignalLog * log);
};


void           gtracer_signal_log_init            (GTracerSignalLog * log, 
						   GQuark log_type, 
						   GTracerLogProtectedOperations * lops,
						   gpointer instance,
						   GSList* backtrace, 
						   GTracerSignalLogOperations *slops);
void           gtracer_signal_log_fin             (GTracerSignalLog * log);
gpointer       gtracer_signal_log_instance        (GTracerSignalLog * log);
const gchar*   gtracer_signal_log_class_name      (GTracerSignalLog * log);
const gchar*   gtracer_signal_log_signal          (GTracerSignalLog * log);
GSList*        gtracer_signal_log_backtrace       (GTracerSignalLog * log);

/** gtracer_signal_create_notes
    The returned string must be freed(g_free) in the clients. */
gchar*         gtracer_signal_log_create_notes (GTracerSignalLog * log);

/** gtracer_signal_create_message
    The returned string must be freed(g_free) in the clients. */
gchar*         gtracer_signal_log_create_message (GTracerSignalLog * log);

/* ------------------------------------------------------------------
   Callback
   ------------------------------------------------------------------ */
typedef struct _GTracerCallbackLog GTracerCallbackLog;
gpointer       gtracer_callback_log_handler(GTracerCallbackLog * log);
gboolean       gtracer_callback_log_after(GTracerCallbackLog * log);

/* ------------------------------------------------------------------
   Connect
   ------------------------------------------------------------------ */
typedef struct _GTracerConnectLog   GTracerConnectLog;

gpointer       gtracer_connect_log_handler       (GTracerConnectLog * log);
GConnectFlags  gtracer_connect_log_flags         (GTracerConnectLog * log);
gulong         gtracer_connect_log_handler_id    (GTracerConnectLog * log);
void           gtracer_connect_log_install_stock (GTracerConnectLog * log);

/* ------------------------------------------------------------------
   Disconnect
   ------------------------------------------------------------------ */
typedef struct _GTracerDisconnectLog GTracerDisconnectLog;
gulong         gtracer_disconnect_log_handler_id (GTracerDisconnectLog * log);
gpointer       gtracer_disconnect_log_handler    (GTracerDisconnectLog * log);

/* ------------------------------------------------------------------
   Emit
   ------------------------------------------------------------------ */
typedef struct _GTracerEmitLog GTracerEmitLog;
gboolean       gtracer_emit_log_after  (GTracerEmitLog * log);

G_END_DECLS

#endif /* Not def: __GTRACER_SIGNAL_H__ */

/* arch-tag: 01f37062-f781-4749-8cb7-d10acf9d63ad
   (do not change this comment) */
