/* signal_viewer.c --- signal log viewer

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "signal_service.h"
#include "service.h"
#include "plugin.h"
#include "utils.h"

#include "i18n.h"

#include <gtk/gtk.h>

#include <ginspector/ginspector-tag.h>
#include <ginspector/ginspector-gtk-tag.h>
#include <ginspector/ginspector-procedure.h>

#include <stdio.h>

static void gtracer_callback_viewer_func (GTracerLogProtected * log,
					  gpointer viewer_data,
					  GTracerProxyLog * proxy_log);
static void gtracer_callback_viewer_doc  (GtkWidget * widget,
					  GQuark log_type,
					  gpointer viewer_data);

static void gtracer_connect_viewer_func (GTracerLogProtected * log,
					 gpointer viewer_data,
					 GTracerProxyLog * proxy_log);
static void gtracer_connect_viewer_doc  (GtkWidget * widget,
					 GQuark log_type,
					 gpointer viewer_data);

static void gtracer_disconnect_viewer_func (GTracerLogProtected * log,
					    gpointer viewer_data,
					    GTracerProxyLog * proxy_log);
static void gtracer_disconnect_viewer_doc  (GtkWidget * widget,
					    GQuark log_type,
					 gpointer viewer_data);

static void gtracer_emit_viewer_func (GTracerLogProtected * log,
				      gpointer viewer_data,
				      GTracerProxyLog * proxy_log);
static void gtracer_emit_viewer_doc  (GtkWidget * widget,
				      GQuark log_type,
				      gpointer viewer_data);

typedef struct _GTracerSignalViewer GTracerSignalViewer;
typedef struct _GTracerSignalModel  GTracerSignalModel;

struct _GTracerSignalModel
{
  GtkTreeModel * tree;
  GTracerIterStack * iter_stack;
};

struct _GTracerSignalViewer
{
  GtkWidget * window;
  GtkWidget * view;
  GtkWidget * trace_toggle;
  GtkWidget * clear_button;
  GtkWidget * filter_appearance_toggle;
  GtkWidget * filter_appearance_arrow;
  GtkWidget * statusbar;
  guint context_id;
  
  GTracerSignalModel model;
  
  GtkWidget * filter_box;
  
  GtkWidget * popup_menu;
  gboolean trace;
};

static GTracerSignalViewer * gtracer_signal_viewer_new  (void);
static gpointer              gtracer_signal_viewer_show (gpointer data, gpointer func_data, gpointer invoke_data);

static gboolean              gtracer_signal_viewer_stop_trace  (GtkWidget	     *widget,
								GdkEventAny	     *event,
								gpointer              user_data);

static void                  gtracer_signal_viewer_toggle_trace(GtkToggleButton * button,
								gpointer user_data);
static void                  gtracer_signal_viewer_clear_logs  (GtkButton * button,
								gpointer user_data);
static void                  gtracer_signal_viewer_selection_changed(GtkTreeSelection *selection,
								     gpointer user_data);
static void                  gtracer_signal_viewer_selection_activated(GtkTreeView       *tree_view,
								       GtkTreePath       *path,
								       GtkTreeViewColumn *column,
								       gpointer user_data);
static GtkWidget *           gtracer_signal_viewer_view_new(GTracerSignalViewer * viewer);

static GtkWidget *           gtracer_signal_viewer_menu_new(GtkWidget * target);
static void                  gtracer_signal_viewer_menu_detach(GtkWidget *attach_widget,
							       GtkMenu   *menu) ;


static gboolean              gtracer_signal_viewer_menu_show(GtkWidget         *widget,
							     GdkEventButton    *event,
							     gpointer           user_data);

static void   gtracer_signal_model_init     (GTracerSignalModel * model);
static void   gtracer_signal_model_clear    (GTracerSignalModel * model);
static void   gtracer_signal_model_add_log  (GTracerSignalModel * model, 
					     GTracerSignalLog * log,
					     GTracerProxyLog  * proxy_log);
static void   gtracer_signal_model_deepen   (GTracerSignalModel * model);
static void   gtracer_signal_model_shallow  (GTracerSignalModel * model);
static void   gtracer_signal_model_show_backtrace (GTracerSignalModel * model,
						   GtkTreePath       *path);
static gpointer gtracer_signal_model_get_instance(GTracerSignalModel * model,
						  GtkTreePath       *path);

static void   gtracer_signal_model_extract_time_stamp (GtkTreeViewColumn *col,
						       GtkCellRenderer   *renderer,
						       GtkTreeModel      *model,
						       GtkTreeIter       *iter,
						       gpointer           user_data);
static void   gtracer_signal_model_extract_log_type   (GtkTreeViewColumn *col,
						       GtkCellRenderer   *renderer,
						       GtkTreeModel      *model,
						       GtkTreeIter       *iter,
						       gpointer           user_data);
static void   gtracer_signal_model_extract_class      (GtkTreeViewColumn *col,
						       GtkCellRenderer   *renderer,
						       GtkTreeModel      *model,
						       GtkTreeIter       *iter,
						       gpointer           user_data);
static void   gtracer_signal_model_extract_instance   (GtkTreeViewColumn *col,
						       GtkCellRenderer   *renderer,
						       GtkTreeModel      *model,
						       GtkTreeIter       *iter,
						       gpointer           user_data);
static void   gtracer_signal_model_extract_signal     (GtkTreeViewColumn *col,
						       GtkCellRenderer   *renderer,
						       GtkTreeModel      *model,
						       GtkTreeIter       *iter,
						       gpointer           user_data);
static void   gtracer_signal_model_extract_notes      (GtkTreeViewColumn *col,
						       GtkCellRenderer   *renderer,
						       GtkTreeModel      *model,
						       GtkTreeIter       *iter,
						       gpointer           user_data);
static gchar* gtracer_signal_model_extract_message    (GTracerSignalModel * model,
						       GtkTreeIter *iter);


/* ------------------------------------------------------------------
   Generic
   ------------------------------------------------------------------ */
void
gtracer_signal_viewer_init (GError ** err)
{
  GTracerSignalViewer * viewer;
  
  viewer = gtracer_signal_viewer_new();

  gtracer_viewers_install(GTRACER_CALLBACK_LOG_NAME,
			  gtracer_callback_viewer_func,
			  gtracer_callback_viewer_doc,
			  viewer);
  gtracer_viewers_install(GTRACER_CONNECT_LOG_NAME,
			  gtracer_connect_viewer_func,
			  gtracer_connect_viewer_doc,
			  viewer);
  gtracer_viewers_install(GTRACER_DISCONNECT_LOG_NAME,
			  gtracer_disconnect_viewer_func,
			  gtracer_disconnect_viewer_doc,
			  viewer);
  gtracer_viewers_install(GTRACER_EMIT_LOG_NAME,
			  gtracer_emit_viewer_func,
			  gtracer_emit_viewer_doc,
			  viewer);

  g_inspector_register_procedure ("signal-tracer",
				  "Signal Tracer",
				  G_INSPECTOR_PROCEDURE_TYPE_WINDOW,
				  G_INSPECTOR_ELEMENT_TYPE_NONE,
				  G_TYPE_NONE,
				  gtracer_signal_viewer_show,
				  viewer );
  g_tracer_append_menu_item(_("/View/Signal Tracer"), 
			    NULL, 
			    "signal-tracer");
}

static gpointer
gtracer_signal_viewer_show(gpointer data, gpointer func_data, gpointer invoke_data)
{
  GTracerSignalViewer * viewer = func_data;

  g_return_val_if_fail (viewer, NULL);
  g_return_val_if_fail (viewer->window, NULL);
  gtk_widget_show_all(viewer->window);
  return viewer->window;
}

static void
gtracer_signal_viewer_toggle_trace(GtkToggleButton * button, gpointer user_data)
{
  GTracerSignalViewer * viewer = user_data;
  gboolean active = gtk_toggle_button_get_active(button);
  
  gtk_widget_set_sensitive(viewer->filter_box,
			   !active);
  gtk_widget_set_sensitive(viewer->clear_button,
			   !active);
  
  viewer->trace = active;
}

static gboolean
gtracer_signal_viewer_stop_trace  (GtkWidget	     *widget,
				   GdkEventAny	     *event,
				   gpointer              user_data)
{
  GTracerSignalViewer * viewer = user_data;

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(viewer->trace_toggle), 
			       FALSE);
  return FALSE;			/* propagate */
}

static void
gtracer_signal_viewer_clear_logs  (GtkButton * button, gpointer user_data)
{
  GTracerSignalViewer * viewer = user_data;
  gtracer_signal_model_clear(&viewer->model);
}
  

static GTracerSignalViewer *
gtracer_signal_viewer_new(void)
{
  GTracerSignalViewer * viewer = g_new(GTracerSignalViewer, 1);
  GtkWidget * hbox;
  GtkWidget * scrolled_window;
  GtkWidget * vbox;
#if 0
  GtkWidget * control_vbox;
#endif /* 0 */

#if 0
  GtkWidget * alignment;
#endif /* 0 */
  GtkAdjustment * adjustment;
  
  viewer->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_inspector_tag_to_widget(viewer->window);
  gtk_window_set_title(GTK_WINDOW(viewer->window),
		       _("Signal Tracer"));

  vbox = gtk_vbox_new(FALSE,	/* !homogeneous */
		      4);
  gtk_container_add(GTK_CONTAINER(viewer->window), vbox);

#if 0  
  alignment = gtk_alignment_new(1.0, 0.50, 0.0, 1.0);
  gtk_box_pack_start(GTK_BOX(vbox), 
		     alignment, 
		     FALSE, 	/* expand */
		     FALSE,	/* fill */
		     2);
  viewer->filter_appearance_toggle = gtk_toggle_button_new();
  gtk_container_add(GTK_CONTAINER(alignment),
		    viewer->filter_appearance_toggle);
  viewer->filter_appearance_arrow  = gtk_arrow_new(GTK_ARROW_RIGHT,
						   GTK_SHADOW_NONE);
  gtk_container_add(GTK_CONTAINER(viewer->filter_appearance_toggle),
		    viewer->filter_appearance_arrow);
#endif /* 0 */
  
  hbox = gtk_hbox_new(FALSE,	/* !homogeneous */
		      4);
  gtk_box_pack_start(GTK_BOX(vbox), 
		     hbox, 
		     TRUE, 	/* expand */
		     TRUE,	/* fill */
		     2);
  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				 GTK_POLICY_AUTOMATIC, 
				 GTK_POLICY_AUTOMATIC);
  adjustment 	  = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrolled_window));
  g_inspector_tag_to_object(G_OBJECT(adjustment));
  adjustment 	  = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolled_window));
  g_inspector_tag_to_object(G_OBJECT(adjustment));

  gtk_box_pack_start(GTK_BOX(hbox), 
		     scrolled_window, 
		     TRUE, 	/* expand */
		     TRUE,	/* fill */
		     2);
  
  viewer->view 	       = gtracer_signal_viewer_view_new(viewer);
  gtk_widget_set_size_request (viewer->view, 400, -1);
  
  viewer->popup_menu = gtracer_signal_viewer_menu_new(GTK_WIDGET (viewer->view));
  
  gtracer_signal_model_init(&viewer->model);
  
  gtk_tree_view_set_model (GTK_TREE_VIEW (viewer->view), viewer->model.tree);
  gtk_container_add(GTK_CONTAINER(scrolled_window), viewer->view);

#if 0
  control_vbox = gtk_vbox_new (FALSE, /* !homogeneous */
			       4);
  gtk_box_pack_start(GTK_BOX(hbox), 
		     control_vbox,
		     FALSE,
		     FALSE,
		     2);
#endif /* 0 */  
  viewer->filter_box = gtk_vbox_new (FALSE, /* !homogeneous */
				     4);
  gtk_box_pack_start(GTK_BOX(hbox), 
		     viewer->filter_box, 
		     FALSE, 	/* expand */
		     FALSE,	/* fill */
		     2);
#if 0
  gtk_box_pack_start(GTK_BOX(control_vbox), 
		     viewer->filter_box, 
		     FALSE, 	/* expand */
		     FALSE,	/* fill */
		     2);
#endif /* 0 */

  
  viewer->clear_button = gtk_button_new_with_label(_("Clear"));
  g_signal_connect(G_OBJECT(viewer->clear_button),
		   "clicked",
		   G_CALLBACK(gtracer_signal_viewer_clear_logs),
		   viewer);
#if 0
  gtk_box_pack_end(GTK_BOX(control_vbox),
		   viewer->clear_button,
		   FALSE,
		   FALSE,
		   2);
#endif /* 0 */

  viewer->trace = FALSE;
  viewer->trace_toggle = gtk_toggle_button_new_with_label(_("Trace"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(viewer->trace_toggle),
			       viewer->trace);
  g_signal_connect(G_OBJECT(viewer->trace_toggle),
		   "toggled",
		   G_CALLBACK(gtracer_signal_viewer_toggle_trace),
		   viewer);

#if 0
  gtk_box_pack_end(GTK_BOX(control_vbox),
		   viewer->trace_toggle,
		   FALSE,
		   FALSE,
		   2);
#endif /* 0 */

  
  viewer->statusbar  = gtk_statusbar_new();
  viewer->context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(viewer->statusbar),
						    "Log extra information");
  /* Pushing a dummy string */
  gtk_statusbar_push(GTK_STATUSBAR(viewer->statusbar), 
		     viewer->context_id, "");
  gtk_box_pack_start(GTK_BOX(vbox), 
		     viewer->statusbar, 
		     FALSE, 	/* expand */
		     FALSE,	/* fill */
		     2);
  gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(viewer->statusbar), 
				    FALSE);
  gtk_box_pack_end(GTK_BOX(viewer->statusbar),
		   viewer->trace_toggle,
		   FALSE,
		   FALSE,
		   2);
  gtk_box_pack_end(GTK_BOX(viewer->statusbar),
		   viewer->clear_button,
		   FALSE,
		   FALSE,
		   2);


  g_signal_connect(G_OBJECT(viewer->window),
		   "delete_event",
		   G_CALLBACK(gtracer_signal_viewer_stop_trace),
		   viewer);
  
  g_signal_connect(G_OBJECT(viewer->window),
		   "delete_event",
		   G_CALLBACK(gtk_widget_hide_on_delete),
		   NULL);

  g_signal_connect(G_OBJECT(viewer->view),
		   "button_press_event",
		   G_CALLBACK(gtracer_signal_viewer_menu_show),
		   viewer);

  return viewer;
}

static GtkWidget *
gtracer_signal_viewer_view_new(GTracerSignalViewer * viewer)
{
  GtkWidget * tree_view;
  GtkCellRenderer * text_renderer;
  gint position;
  GtkTreeViewColumn * col;
  GtkTreeSelection *tree_selection;
  
  /* Type, Class, Object, Extra */
  tree_view = gtk_tree_view_new();
  text_renderer = gtk_cell_renderer_text_new ();
  g_inspector_tag_to_object(G_OBJECT(text_renderer));
  
  position = 0;
  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Time"),
							text_renderer,
							gtracer_signal_model_extract_time_stamp,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);
  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Log type"),
							text_renderer,
							gtracer_signal_model_extract_log_type,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Class"),
							text_renderer,
							gtracer_signal_model_extract_class,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Instance"),
							text_renderer,
							gtracer_signal_model_extract_instance,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Signal"),
							text_renderer,
							gtracer_signal_model_extract_signal,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Notes"),
							text_renderer,
							gtracer_signal_model_extract_notes,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);
 
  tree_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
  g_inspector_tag_to_object(G_OBJECT(tree_selection));

  gtk_tree_selection_set_mode (GTK_TREE_SELECTION (tree_selection),
			       GTK_SELECTION_SINGLE);
  g_signal_connect (tree_selection, 
		    "changed", 
		    G_CALLBACK (gtracer_signal_viewer_selection_changed), viewer);
  g_signal_connect (tree_view, 
		    "row_activated", 
		    G_CALLBACK (gtracer_signal_viewer_selection_activated), 
		    viewer);
  return tree_view;
}

static void
gtracer_signal_viewer_selection_changed(GtkTreeSelection *selection,
					gpointer user_data)
{
  GTracerSignalViewer * viewer = user_data;
  GtkTreeModel * model;
  GtkTreeIter iter;
  gchar * msg;
  
  if (gtk_tree_selection_get_selected(selection, &model, &iter) == TRUE)
    {
      gtk_statusbar_pop(GTK_STATUSBAR(viewer->statusbar), viewer->context_id);
      msg = gtracer_signal_model_extract_message(&viewer->model,
						 &iter);
      if (msg)
	{
	  gtk_statusbar_push(GTK_STATUSBAR(viewer->statusbar), 
			     viewer->context_id,
			     msg);
	  g_free(msg);
	}
      else
	gtk_statusbar_push(GTK_STATUSBAR(viewer->statusbar), 
			   viewer->context_id,
			   "");
    }
}

static void
gtracer_signal_viewer_selection_activated (GtkTreeView       *tree_view,
					   GtkTreePath       *path,
					   GtkTreeViewColumn *column,
					   gpointer user_data)
{
  GTracerSignalViewer * viewer = user_data;


  g_return_if_fail (viewer);

  gtracer_signal_model_show_backtrace(&viewer->model, path);
}

static GtkWidget*
gtracer_signal_viewer_menu_new(GtkWidget * target)
{
  GtkWidget * menu = gtk_menu_new ();

  g_inspector_tag_to_widget (gtk_widget_get_toplevel (menu));
  gtk_menu_attach_to_widget (GTK_MENU (menu),
			     target,
			     gtracer_signal_viewer_menu_detach);
  return menu;
}

static void
gtracer_signal_viewer_menu_detach(GtkWidget *attach_widget,
				  GtkMenu   *menu) 

{
  /* This is a workaround for the gtk+'s bug.  */
}

static gboolean
gtracer_signal_viewer_menu_show(GtkWidget         *widget,
				GdkEventButton    *event,
				gpointer           user_data)
{
  GTracerSignalViewer * viewer;
  gpointer           instance;
  GtkTreeView       *tree_view;
  GtkTreeViewColumn *tree_view_column;
  GtkTreePath       *tree_path;
  gboolean r;
  
  tree_view  = GTK_TREE_VIEW (widget);
  viewer     = user_data;

  
  if (event->button != 3)
    return FALSE;
  
  r = gtk_tree_view_get_path_at_pos (tree_view,
                                     event->x,
				     event->y,
                                     &tree_path,
				     &tree_view_column,
				     NULL,
				     NULL);
  if (r == FALSE)
    return FALSE;

  instance = gtracer_signal_model_get_instance(&viewer->model, tree_path);
  g_inspector_invoke_procedure_by_name("popup-menu-for-gtypeinstance",
				       instance,
				       event);
  return FALSE;
}  


/* ------------------------------------------------------------------
   Model
   ------------------------------------------------------------------ */
enum
{
  COL_LOG = 0,
  COL_PROXY = 1,
  NUM_COLS
};

static void
gtracer_signal_model_init     (GTracerSignalModel * model)
{
  GtkTreeStore * store;
  model->iter_stack = gtracer_iter_stack_new();
  store       = gtk_tree_store_new (NUM_COLS, GTracerLogType, GTracerLogType); 
  model->tree = GTK_TREE_MODEL(store);
  g_inspector_tag_to_object(G_OBJECT(model->tree));
}

static void
gtracer_signal_model_clear(GTracerSignalModel * model)
{
  gtk_tree_store_clear(GTK_TREE_STORE(model->tree));
  gtracer_iter_stack_clear(model->iter_stack);
}

static void
gtracer_signal_model_add_log  (GTracerSignalModel * model, 
			       GTracerSignalLog * log,
			       GTracerProxyLog  * proxy_log)
{
  GtkTreeIter * current_iter;
  GtkTreeIter * parent_iter;
  
  current_iter = gtracer_iter_stack_current_iter(model->iter_stack);
  parent_iter  = gtracer_iter_stack_parent_iter(model->iter_stack);
  
  gtk_tree_store_append(GTK_TREE_STORE(model->tree),
			current_iter,
			parent_iter);
  gtk_tree_store_set (GTK_TREE_STORE(model->tree), 
		      current_iter, 
		      COL_LOG, log, 
		      COL_PROXY, proxy_log,
		      -1);
}

static void
gtracer_signal_model_deepen   (GTracerSignalModel * model)
{
  gtracer_iter_stack_push_alloc(model->iter_stack);
}

static void
gtracer_signal_model_shallow  (GTracerSignalModel * model)
{
  gtracer_iter_stack_pop(model->iter_stack);  
}

static gpointer
gtracer_signal_model_get_instance(GTracerSignalModel * model,
				  GtkTreePath       *path)
{
  GTracerLog * log;
  GtkTreeIter  iter;
  gpointer instance;
  if (gtk_tree_model_get_iter(GTK_TREE_MODEL(model->tree), &iter, path) == FALSE)
    return NULL;
  
  gtk_tree_model_get(model->tree, &iter, COL_LOG, &log, -1);
  g_return_val_if_fail (log, NULL);
  
  instance = gtracer_signal_log_instance((GTracerSignalLog *)log);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */

  /* TODO: Autounref
     instance = g_object_ref(instance);   */
  
  return instance;
}

static void
gtracer_signal_model_show_backtrace (GTracerSignalModel * model,
				     GtkTreePath       *path)
{
  GTracerSignalLog * log;
  GtkTreeIter  iter;
  GSList * bt;
  
  if (gtk_tree_model_get_iter(GTK_TREE_MODEL(model->tree), &iter, path) == FALSE)
    return;

  gtk_tree_model_get(model->tree, &iter, COL_LOG, &log, -1);
  g_return_if_fail(log);

  bt = gtracer_signal_log_backtrace(log);
  if (bt)
    gtrace_backtrace_show_backtrace(NULL, bt);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
}

static void
gtracer_signal_model_extract_time_stamp (GtkTreeViewColumn *col,
					 GtkCellRenderer   *renderer,
					 GtkTreeModel      *model,
					 GtkTreeIter       *iter,
					 gpointer           user_data)
{
  GTracerLogProtected * log = NULL;
  GTracerLogProtected * log_proxy = NULL;
  glong d;
  gchar * ts_string;
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  gtk_tree_model_get(model, iter, COL_PROXY, &log_proxy, -1);
  g_return_if_fail (log);

  if (log_proxy) 
    d = log_proxy->time_stamp.tv_sec - gtracer_epoch.tv_sec;
  else
    d = log->time_stamp.tv_sec - gtracer_epoch.tv_sec;

  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
  if (log_proxy) 
    g_boxed_free(GTracerLogType, log_proxy); /* MODEL_GET_FREE */
  
  ts_string = g_strdup_printf("%ld", d);
  g_object_set(renderer, "text", ts_string, NULL); 
  g_free(ts_string);
}

static void
gtracer_signal_model_extract_log_type   (GtkTreeViewColumn *col,
					 GtkCellRenderer   *renderer,
					 GtkTreeModel      *model,
					 GtkTreeIter       *iter,
					 gpointer           user_data)
{
  GTracerLog * log;
  const gchar * type_name;
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  type_name = gtracer_log_type_name(log);
  g_object_set(renderer, "text", type_name, NULL);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
}

static void
gtracer_signal_model_extract_class  (GtkTreeViewColumn *col,
				     GtkCellRenderer   *renderer,
				     GtkTreeModel      *model,
				     GtkTreeIter       *iter,
				     gpointer           user_data)
{
  GTracerLog * log;
  const gchar * class_name;
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  class_name = gtracer_signal_log_class_name((GTracerSignalLog *)log);
  g_object_set(renderer, "text", class_name, NULL);
  g_boxed_free(GTracerLogType, log);/* MODEL_GET_FREE */
}

static void
gtracer_signal_model_extract_instance (GtkTreeViewColumn *col,
				       GtkCellRenderer   *renderer,
				       GtkTreeModel      *model,
				       GtkTreeIter       *iter,
				       gpointer           user_data)
{
  GTracerLog * log;
  gpointer instance;
#define BUFFER_LENGTH 32
  gchar buffer[BUFFER_LENGTH];

  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  instance = gtracer_signal_log_instance((GTracerSignalLog*)log);
  snprintf(buffer, BUFFER_LENGTH, "%p", instance);  
  g_object_set(renderer, "text", buffer, NULL);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */

}

static void
gtracer_signal_model_extract_signal (GtkTreeViewColumn *col,
				     GtkCellRenderer   *renderer,
				     GtkTreeModel      *model,
				     GtkTreeIter       *iter,
				     gpointer           user_data)
{
  GTracerLog * log;
  const gchar * signal;

  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  signal = gtracer_signal_log_signal((GTracerSignalLog*)log);
  g_object_set(renderer, "text", signal? signal: _("{lost}"), NULL); 
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
}

static void
gtracer_signal_model_extract_notes      (GtkTreeViewColumn *col,
					 GtkCellRenderer   *renderer,
					 GtkTreeModel      *model,
					 GtkTreeIter       *iter,
					 gpointer           user_data)
{
  GTracerLog * log;
  gchar * notes;

  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  notes = gtracer_signal_log_create_notes((GTracerSignalLog*)log);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
  g_object_set(renderer, "text", notes? notes: "", NULL);
  if (notes) 
    g_free(notes);
}

static gchar*
gtracer_signal_model_extract_message    (GTracerSignalModel * model,
					 GtkTreeIter *iter)
{
  GTracerLogProtected * log = NULL;
  gchar * msg;

  gtk_tree_model_get(model->tree, iter, COL_LOG, &log, -1);
  
  msg = gtracer_signal_log_create_message((GTracerSignalLog *)log);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */

  return msg;
}


/* ------------------------------------------------------------------
   Callback
   ------------------------------------------------------------------ */
static void
gtracer_callback_viewer_func(GTracerLogProtected * log, 
			     gpointer viewer_data,
			     GTracerProxyLog * proxy_log)
{
  GTracerSignalViewer * viewer = viewer_data;
  
  if (viewer->trace == FALSE)
    return ;

  if (FALSE == gtracer_callback_log_after((GTracerCallbackLog *)log)) {
    gtracer_signal_model_add_log(&viewer->model, 
				 (GTracerSignalLog *)log,
				 proxy_log);
    gtracer_signal_model_deepen(&viewer->model);
  } else
    gtracer_signal_model_shallow(&viewer->model);
  /* gtracer_log_dump(log); */
}

static void
gtracer_callback_viewer_doc  (GtkWidget * widget,
			      GQuark log_type,
			      gpointer viewer_data)
{
    GTracerSignalViewer * viewer = viewer_data;

    gtk_box_pack_start(GTK_BOX(viewer->filter_box), widget, FALSE, FALSE, 4);
}

/* ------------------------------------------------------------------
   Connect
   ------------------------------------------------------------------ */
static void
gtracer_connect_viewer_func(GTracerLogProtected * log, 
			    gpointer viewer_data,
			    GTracerProxyLog * proxy_log)
{
  GTracerSignalViewer * viewer = viewer_data;
  
  if (viewer->trace)
    gtracer_signal_model_add_log(&viewer->model, 
				 (GTracerSignalLog *)log,
				 proxy_log);
  /* gtracer_log_dump(log); */
}

static void
gtracer_connect_viewer_doc  (GtkWidget * widget,
			      GQuark log_type,
			      gpointer viewer_data)
{
    GTracerSignalViewer * viewer = viewer_data;

    gtk_box_pack_start(GTK_BOX(viewer->filter_box), widget, FALSE, FALSE, 4);
}


/* ------------------------------------------------------------------
   Disconnect
   ------------------------------------------------------------------ */
static void
gtracer_disconnect_viewer_func (GTracerLogProtected * log,
				gpointer viewer_data,
				GTracerProxyLog * proxy_log)
{
  GTracerSignalViewer * viewer = viewer_data;
  
  if (viewer->trace)
    gtracer_signal_model_add_log(&viewer->model, 
				 (GTracerSignalLog *)log,
				 proxy_log);
  /* gtracer_log_dump(log); */
}


static void
gtracer_disconnect_viewer_doc  (GtkWidget * widget,
				GQuark log_type,
				gpointer viewer_data)
{
    GTracerSignalViewer * viewer = viewer_data;

    gtk_box_pack_start(GTK_BOX(viewer->filter_box), widget, FALSE, FALSE, 4);
}


/* ------------------------------------------------------------------
   Emit
   ------------------------------------------------------------------ */
static void
gtracer_emit_viewer_func (GTracerLogProtected * log, 
			  gpointer viewer_data,
			  GTracerProxyLog * proxy_log)
{
  GTracerSignalViewer * viewer = viewer_data;

  if (viewer->trace == FALSE) 
    return ;
  
  if (FALSE == gtracer_callback_log_after((GTracerCallbackLog *)log)) {
    gtracer_signal_model_add_log(&viewer->model, 
				 (GTracerSignalLog *)log,
				 proxy_log);
    gtracer_signal_model_deepen(&viewer->model);
  } else
    gtracer_signal_model_shallow(&viewer->model);
  /* gtracer_log_dump(log); */
  /* gtracer_log_dump(log); */
}

static void
gtracer_emit_viewer_doc  (GtkWidget * widget,
			  GQuark log_type,
			  gpointer viewer_data)
{
  GTracerSignalViewer * viewer = viewer_data;

  gtk_box_pack_start(GTK_BOX(viewer->filter_box), widget, FALSE, FALSE, 4);
}

/* arch-tag: 47f339b6-8629-4f31-8cfe-5237ea7126c4
   (do not change this comment) */
