/* source_filter.c --- source log filter

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "source_service.h"
#include "service.h"
#include "plugin.h"

static gboolean gtracer_source_filter_func   (GTracerLogProtected * log,
					      gpointer filter_data,
					      GTracerProxyLog * proxy_log);
static gboolean gtracer_dropsrc_filter_func  (GTracerLogProtected * log,
					      gpointer filter_data,
					      GTracerProxyLog * proxy_log);

static guint gtracer_source_filter_last_idle_tag    = 0;
static guint gtracer_source_filter_last_timeout_tag = 0;

void
gtracer_source_filter_init (GError ** err)
{
  gtracer_filters_install(GTRACER_IDLE_LOG_NAME,
			  gtracer_source_filter_func,
			  NULL,
			  &gtracer_source_filter_last_idle_tag);
  gtracer_filters_install(GTRACER_TIMEOUT_LOG_NAME,
			  gtracer_source_filter_func,
			  NULL,
			  &gtracer_source_filter_last_timeout_tag);
  gtracer_filters_install(GTRACER_DROPSRC_LOG_NAME,
			  gtracer_dropsrc_filter_func,
			  NULL,
			  NULL);
}

static gboolean
gtracer_source_filter_func   (GTracerLogProtected * log,
			      gpointer filter_data,
			      GTracerProxyLog * proxy_log)
{
  GTracerSourceLog * source_log = (GTracerSourceLog *)log;
  gpointer function             = gtracer_source_log_function(source_log);
  gboolean accept 		= !gtracer_check_function(function);
  
  guint    tag 			= gtracer_source_log_tag(source_log);
  guint*   cache                = (guint *)filter_data;
  if (!accept) 
    (*cache) = tag;
  
  /* g_message("gtracer_source_filter_func accept : %d, tag: %u", accept, tag); */
  return accept;
}

static gboolean
gtracer_dropsrc_filter_func  (GTracerLogProtected * log,
			      gpointer filter_data,
			      GTracerProxyLog * proxy_log)
{
  GTracerDropsrcLog * dropsrc_log = (GTracerDropsrcLog *)log;
  guint tag 			  = gtracer_dropsrc_log_tag (dropsrc_log);
  gboolean accept;

  if (tag    == gtracer_source_filter_last_timeout_tag
      || tag == gtracer_source_filter_last_idle_tag)
    accept = FALSE;
  else
    accept = !gtracer_check_tag(tag);

  /* g_message("gtracer_dropsrc_filter_func accept : %d, tag: %u", accept, tag); */
  return accept;
}

/* arch-tag: d7fda112-8384-4d68-a4d4-5c4359ff37b6
   (do not change this comment) */
