/* source_viewer.c --- GSource log viewer

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "service.h"
#include "source_service.h"
#include "utils.h"

#include "i18n.h"
#include <ginspector/ginspector-tag.h>
#include <ginspector/ginspector-gtk-tag.h>
#include <ginspector/ginspector-procedure.h>


static void gtracer_source_viewer_func  (GTracerLogProtected * log,
					 gpointer viewer_data,
					 GTracerProxyLog * proxy_log);
static void gtracer_dropsrc_viewer_func (GTracerLogProtected * log,
					 gpointer viewer_data,
					 GTracerProxyLog * proxy_log);

typedef struct _GTracerSourceViewer GTracerSourceViewer;
struct _GTracerSourceViewer
{
  GtkWidget * window;
  GtkWidget * view;
  GtkWidget * statusbar;
  
  guint context_id;

  GtkTreeModel * list_store;
};

static GTracerSourceViewer * gtracer_source_viewer_new        (void);
static gpointer              gtracer_source_viewer_show       (gpointer data, gpointer func_data, gpointer invoke_data);
static GtkWidget*            gtracer_source_viewer_view_new   (GTracerSourceViewer * viewer);
static void                  gtracer_source_viewer_selection_changed(GtkTreeSelection *selection,
								     gpointer user_data);
static void                  gtracer_source_viewer_selection_activated(GtkTreeView       *tree_view,
								       GtkTreePath       *path,
								       GtkTreeViewColumn *column,
								       gpointer user_data);

static GtkTreeModel* gtracer_source_model_new                 (void);
static void          gtracer_source_model_add_log             (GtkTreeModel * model,
							       GTracerSourceLog * log);
static void          gtracer_source_model_remove_log           (GtkTreeModel * model,
								guint tag);
static gboolean      gtracer_source_model_remove_log_helper (GtkTreeModel * model,
							     GtkTreePath  * path,
							     GtkTreeIter  * iter,
							     gpointer data);


static void          gtracer_source_model_extract_time_stamp  (GtkTreeViewColumn *col,
							       GtkCellRenderer   *renderer,
							       GtkTreeModel      *model,
							       GtkTreeIter       *iter,
							       gpointer           user_data);
static void          gtracer_source_model_extract_source_type (GtkTreeViewColumn *col,
							       GtkCellRenderer   *renderer,
							       GtkTreeModel      *model,
							       GtkTreeIter       *iter,
							       gpointer           user_data);
static void          gtracer_source_model_extract_tag         (GtkTreeViewColumn *col,
							       GtkCellRenderer   *renderer,
							       GtkTreeModel      *model,
							       GtkTreeIter       *iter,
							       gpointer           user_data);
static void          gtracer_source_model_extract_priority    (GtkTreeViewColumn *col,
							       GtkCellRenderer   *renderer,
							       GtkTreeModel      *model,
							       GtkTreeIter       *iter,
							       gpointer           user_data);
static void          gtracer_source_model_extract_interval    (GtkTreeViewColumn *col,
							       GtkCellRenderer   *renderer,
							       GtkTreeModel      *model,
							       GtkTreeIter       *iter,
							       gpointer           user_data);
static void          gtracer_source_model_extract_function    (GtkTreeViewColumn *col,
							       GtkCellRenderer   *renderer,
							       GtkTreeModel      *model,
							       GtkTreeIter       *iter,
							       gpointer           user_data);
static gchar*        gtracer_source_model_extract_message    (GtkTreeModel * model,
							      GtkTreeIter *iter);
static void         gtracer_source_model_show_backtrace      (GtkTreeModel * model,
							      GtkTreePath  * path);


/* ------------------------------------------------------------------
   Viewer
   ------------------------------------------------------------------ */
void
gtracer_source_viewer_init (GError ** err)
{
  GTracerSourceViewer * viewer;

  viewer = gtracer_source_viewer_new();
  
  gtracer_viewers_install(GTRACER_IDLE_LOG_NAME,
			  gtracer_source_viewer_func,
			  NULL,
			  viewer);
  gtracer_viewers_install(GTRACER_TIMEOUT_LOG_NAME,
			  gtracer_source_viewer_func,
			  NULL,
			  viewer);
  gtracer_viewers_install(GTRACER_DROPSRC_LOG_NAME,
			  gtracer_dropsrc_viewer_func,
			  NULL,
			  viewer);
  /* TODO */
  g_inspector_register_procedure("idle-and-timeout",
				 "Idle and Timeout",
				 G_INSPECTOR_PROCEDURE_TYPE_WINDOW,
				 G_INSPECTOR_ELEMENT_TYPE_NONE,
				 G_TYPE_NONE,
				 gtracer_source_viewer_show,
				 viewer);
  g_tracer_append_menu_item(_("/View/Idle and Timeout"), 
			    NULL, 
			    "idle-and-timeout");
}

static GTracerSourceViewer *
gtracer_source_viewer_new  (void)
{
  GTracerSourceViewer * viewer = g_new(GTracerSourceViewer, 1);
  GtkWidget * scrolled_window;
  GtkWidget * vbox;
  GtkAdjustment * adjustment;
  
  viewer->window 	       = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  g_inspector_tag_to_widget(viewer->window);
  gtk_window_set_title(GTK_WINDOW(viewer->window),
		       _("Idle and Timeout"));
  
  vbox = gtk_vbox_new(FALSE,	/* !homogeneous */
		      4);
  gtk_container_add(GTK_CONTAINER(viewer->window), vbox);


  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				 GTK_POLICY_NEVER, 
				 GTK_POLICY_AUTOMATIC);
  adjustment 	  = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrolled_window));
  g_inspector_tag_to_object(G_OBJECT(adjustment));
  adjustment 	  = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolled_window));
  g_inspector_tag_to_object(G_OBJECT(adjustment));
  gtk_box_pack_start(GTK_BOX(vbox), 
		     scrolled_window, 
		     TRUE, 	/* expand */
		     TRUE,	/* fill */
		     2);


  viewer->view = gtracer_source_viewer_view_new(viewer);
  viewer->list_store = gtracer_source_model_new();
  gtk_tree_view_set_model (GTK_TREE_VIEW (viewer->view), viewer->list_store);
  
  viewer->statusbar  = gtk_statusbar_new();
  viewer->context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(viewer->statusbar),
						    "Log extra information");
  gtk_widget_set_size_request (viewer->view, -1, 200);
  gtk_container_add(GTK_CONTAINER(scrolled_window), 
		    viewer->view);


  gtk_statusbar_push(GTK_STATUSBAR(viewer->statusbar), 
		     viewer->context_id, "");
  gtk_box_pack_start(GTK_BOX(vbox), 
		     viewer->statusbar, 
		     FALSE, 	/* expand */
		     FALSE,	/* fill */
		     2);
  gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(viewer->statusbar), 
				    FALSE);
  g_signal_connect(G_OBJECT(viewer->window),
		   "delete_event",
		   G_CALLBACK(gtk_widget_hide_on_delete),
		   NULL);

  return viewer;
}

static gpointer
gtracer_source_viewer_show (gpointer data, gpointer func_data, gpointer invoke_data)
{
  GTracerSourceViewer * viewer = func_data;

  g_return_val_if_fail (viewer, NULL);
  g_return_val_if_fail (viewer->window, NULL);
  gtk_widget_show_all(viewer->window);
  
  return viewer->window;
}

static GtkWidget *
gtracer_source_viewer_view_new(GTracerSourceViewer * viewer)
{
  GtkWidget * tree_view;
  GtkCellRenderer * text_renderer;
  gint position;
  GtkTreeViewColumn * col;
  GtkTreeSelection * tree_selection;

  /* Type, Class, Object, Extra */
  tree_view = gtk_tree_view_new();
  text_renderer = gtk_cell_renderer_text_new ();
  g_inspector_tag_to_object(G_OBJECT(text_renderer));
  
  position = 0;
  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Time"),
							text_renderer,
							gtracer_source_model_extract_time_stamp,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Source Type"),
							text_renderer,
							gtracer_source_model_extract_source_type,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Tag"),
							text_renderer,
							gtracer_source_model_extract_tag,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Priority"),
							text_renderer,
							gtracer_source_model_extract_priority,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Interval"),
							text_renderer,
							gtracer_source_model_extract_interval,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree_view),
							position,
							_("Function"),
							text_renderer,
							gtracer_source_model_extract_function,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), position - 1);
  gtracer_register_tree_view_column(col);
 
  tree_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
  g_inspector_tag_to_object(G_OBJECT(tree_selection));

  gtk_tree_selection_set_mode (GTK_TREE_SELECTION (tree_selection),
			       GTK_SELECTION_SINGLE);
  g_signal_connect (tree_selection, 
		    "changed", 
		    G_CALLBACK (gtracer_source_viewer_selection_changed), 
		    viewer);
  g_signal_connect (tree_view, 
		    "row_activated", 
		    G_CALLBACK (gtracer_source_viewer_selection_activated), 
		    viewer);
  return tree_view;
}

static void
gtracer_source_viewer_func  (GTracerLogProtected * log,
			     gpointer viewer_data,
			     GTracerProxyLog * proxy_log)
{
  GTracerSourceViewer * viewer = viewer_data;
  
  gtracer_source_model_add_log(viewer->list_store, 
			       (GTracerSourceLog *)log);
}

static void
gtracer_dropsrc_viewer_func (GTracerLogProtected * log,
			     gpointer viewer_data,
			     GTracerProxyLog * proxy_log)
{
  GTracerSourceViewer * viewer = viewer_data;
  GTracerDropsrcLog * dropsrc_log = (GTracerDropsrcLog *)log;
  guint tag;

  g_return_if_fail (log);
  tag = gtracer_dropsrc_log_tag(dropsrc_log);
  gtracer_source_model_remove_log(viewer->list_store, tag);
}

static void
gtracer_source_viewer_selection_changed(GtkTreeSelection *selection,
					gpointer user_data)
{
  GTracerSourceViewer * viewer = user_data;
  GtkTreeModel * model;
  GtkTreeIter iter;
  gchar * msg;
  
  if (gtk_tree_selection_get_selected(selection, &model, &iter) == TRUE)
    {
      gtk_statusbar_pop(GTK_STATUSBAR(viewer->statusbar), viewer->context_id);
      msg = gtracer_source_model_extract_message(viewer->list_store,
						 &iter);
      if (msg)
	{
	  gtk_statusbar_push(GTK_STATUSBAR(viewer->statusbar), 
			     viewer->context_id,
			     msg);
	  g_free(msg);
	}
      else
	gtk_statusbar_push(GTK_STATUSBAR(viewer->statusbar), 
			   viewer->context_id,
			   "");
    }
}

static void
gtracer_source_viewer_selection_activated(GtkTreeView       *tree_view,
					  GtkTreePath       *path,
					  GtkTreeViewColumn *column,
					  gpointer user_data)
{
  GtkTreeModel * model;

  model = gtk_tree_view_get_model(tree_view);
  gtracer_source_model_show_backtrace(model, path);
}

/* ------------------------------------------------------------------
   Model
   ------------------------------------------------------------------ */
enum
{
  COL_LOG = 0,
  NUM_COLS
};

static GtkTreeModel*
gtracer_source_model_new(void)
{
  GtkListStore * store;
  store = gtk_list_store_new(NUM_COLS, GTracerLogType);
  g_inspector_tag_to_object(G_OBJECT(store));
  return GTK_TREE_MODEL(store);
}

static void
gtracer_source_model_add_log              (GtkTreeModel * model,
					   GTracerSourceLog * log)
{
  GtkListStore* store = GTK_LIST_STORE(model);
  GtkTreeIter   iter;
  gtk_list_store_append (store, &iter);

  gtk_list_store_set (store, &iter, COL_LOG, log, -1);
}

static void
gtracer_source_model_remove_log           (GtkTreeModel * model,
					   guint tag)
{
  gpointer io = GUINT_TO_POINTER(tag);
  gtk_tree_model_foreach(GTK_TREE_MODEL(model),
			 gtracer_source_model_remove_log_helper,
			 &io);

  
  if (io == GUINT_TO_POINTER(tag))
    return ;			/* Cannot find iter to remove */
 
  gtk_list_store_remove(GTK_LIST_STORE(model), io);
  gtk_tree_iter_free(io);
}

static gboolean
gtracer_source_model_remove_log_helper (GtkTreeModel * model,
					GtkTreePath  * path,
					GtkTreeIter  * iter,
					gpointer data)
{
  guint tag = GPOINTER_TO_UINT((*(gpointer *)data));
  GTracerSourceLog * log;
  gboolean result;
  
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);

  if (gtracer_source_log_tag(log) != tag)
    {
      result = FALSE;		/* continue */
      goto out;
    }

  (*(gpointer *)data) = gtk_tree_iter_copy(iter);
  result = TRUE;
  
 out:
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
  return result;
}

static void
gtracer_source_model_extract_time_stamp  (GtkTreeViewColumn *col,
					  GtkCellRenderer   *renderer,
					  GtkTreeModel      *model,
					  GtkTreeIter       *iter,
					  gpointer           user_data)
{
  GTracerLogProtected * log = NULL;
  glong d;
  gchar * ts_string;
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  g_return_if_fail (log);

  d = log->time_stamp.tv_sec - gtracer_epoch.tv_sec;
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
  
  ts_string = g_strdup_printf("%ld", d);
  g_object_set(renderer, "text", ts_string, NULL); 
  g_free(ts_string);
}

static void
gtracer_source_model_extract_source_type (GtkTreeViewColumn *col,
					  GtkCellRenderer   *renderer,
					  GtkTreeModel      *model,
					  GtkTreeIter       *iter,
					  gpointer           user_data)
{
  GTracerLog * log;
  const gchar * type_name;
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  type_name = gtracer_log_type_name(log);
  g_object_set(renderer, "text", type_name, NULL);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
}

static void
gtracer_source_model_extract_tag         (GtkTreeViewColumn *col,
					  GtkCellRenderer   *renderer,
					  GtkTreeModel      *model,
					  GtkTreeIter       *iter,
					  gpointer           user_data)
{
  GTracerSourceLog * log = NULL;
#define BUFFER_LENGTH 32
  gchar buffer[BUFFER_LENGTH];
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  g_return_if_fail (log);
  snprintf(buffer, BUFFER_LENGTH, "%u", 
	   gtracer_source_log_tag(log));  
  g_object_set(renderer, "text", buffer, NULL);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
}

static void
gtracer_source_model_extract_priority    (GtkTreeViewColumn *col,
					  GtkCellRenderer   *renderer,
					  GtkTreeModel      *model,
					  GtkTreeIter       *iter,
					  gpointer           user_data)
{
  GTracerSourceLog * log = NULL;
#define BUFFER_LENGTH 32
  gchar buffer[BUFFER_LENGTH];
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  g_return_if_fail (log);
  snprintf(buffer, BUFFER_LENGTH, "%d", gtracer_source_log_priority(log));  
  g_object_set(renderer, "text", buffer, NULL); 
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
}

static void
gtracer_source_model_extract_interval    (GtkTreeViewColumn *col,
					  GtkCellRenderer   *renderer,
					  GtkTreeModel      *model,
					  GtkTreeIter       *iter,
					  gpointer           user_data)
{
  GTracerSourceLog * log = NULL;
#define BUFFER_LENGTH 32
  gchar buffer[BUFFER_LENGTH];
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  g_return_if_fail (log);
  snprintf(buffer, BUFFER_LENGTH, "%u", gtracer_source_log_interval(log));  
  g_object_set(renderer, "text", buffer, NULL);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
}

static void
gtracer_source_model_extract_function    (GtkTreeViewColumn *col,
					  GtkCellRenderer   *renderer,
					  GtkTreeModel      *model,
					  GtkTreeIter       *iter,
					  gpointer           user_data)
{
  GTracerSourceLog * log = NULL;
  gpointer function;
  AddrslvSymbolInfo * symbol_info;
  gchar * function_name;
  
  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  g_return_if_fail (log);

  function    = gtracer_source_log_function(log);
  symbol_info = gtracer_pmaps_address_resolve(GPOINTER_TO_UINT(function), NULL);
  
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
  
  if (symbol_info)
    function_name = symbol_info->funcname;
  else
    function_name = _("{static, no debug information}");
  g_object_set(renderer, "text", function_name, NULL); 
}

static gchar*
gtracer_source_model_extract_message    (GtkTreeModel * model,
					 GtkTreeIter *iter)
{
  GTracerSourceLog * log = NULL;
  gpointer function;
  gchar * msg;
  AddrslvSymbolInfo * symbol_info;
  GError  * err = NULL;

  gtk_tree_model_get(model, iter, COL_LOG, &log, -1);
  g_return_val_if_fail (log, NULL);
  
  function    = gtracer_source_log_function(log);

  symbol_info = gtracer_pmaps_address_resolve(GPOINTER_TO_UINT(function), 
					      &err);
  if (err) 
    {
      msg = g_strdup_printf(_("Error[symbol]: %s"), err->message);
      g_clear_error(&err);
      goto out;
    }

  msg = gtracer_pmaps_message_new(symbol_info, GPOINTER_TO_UINT(function));

 out:
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
  return msg;
}

static void
gtracer_source_model_show_backtrace      (GtkTreeModel * model,
					  GtkTreePath  * path)
{
  GTracerSourceLog * log;
  GtkTreeIter  iter;
  GSList * bt;
  
  if (gtk_tree_model_get_iter(model, &iter, path) == FALSE)
    return;
  
  gtk_tree_model_get(model, &iter, COL_LOG, &log, -1);
  g_return_if_fail(log);

  bt = gtracer_source_log_backtrace(log);
  if (bt) 
    gtrace_backtrace_show_backtrace(NULL, bt);
  g_boxed_free(GTracerLogType, log); /* MODEL_GET_FREE */
}


/* arch-tag: f4d48c99-fefe-443b-966e-1413d332cc87
   (do not change this comment) */
