/* usignal_viewer.c --- Unix signal viewer

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "usignal_service.h"
#include "plugin.h"
#include "utils.h"

#include "i18n.h"

#include <gtk/gtk.h>
#include <ginspector/ginspector-tag.h>
#include <ginspector/ginspector-gtk-tag.h>
#include <ginspector/ginspector-procedure.h>


typedef struct _GTracerUSignalViewer GTracerUSignalViewer;
struct _GTracerUSignalViewer
{
  GtkWidget * window;
  GtkWidget * view;
  
  GtkWidget * statusbar;
  guint context_id;
};

static GTracerUSignalViewer * gtracer_usignal_viewer_new    (void);
static gpointer               gtracer_usignal_viewer_show   (gpointer data, gpointer func_data, gpointer invoke_data);
static void                   gtracer_usignal_viewer_update (GTracerUSignalViewer * viewer);

static void                  gtracer_usignal_viewer_selection_changed(GtkTreeSelection *selection,
								      gpointer user_data);

void
gtracer_usignal_viewer_init(GError ** err)
{
  GTracerUSignalViewer * viewer;

  viewer = gtracer_usignal_viewer_new();

  g_inspector_register_procedure ("unix-signals",
				  "Unix Signals",
				  G_INSPECTOR_PROCEDURE_TYPE_WINDOW,
				  G_INSPECTOR_ELEMENT_TYPE_NONE,
				  G_TYPE_NONE,
				  gtracer_usignal_viewer_show,
				  viewer);

  g_tracer_append_menu_item(_("/View/Unix Signals"), 
			    NULL, 
			    "unix-signals");
}

static GTracerUSignalViewer *
gtracer_usignal_viewer_new(void)
{
  GTracerUSignalViewer * viewer;
  GtkWidget * vbox;
  GtkWidget * scrolled_window;
  GtkAdjustment * adjustment;
  GtkCellRenderer * text_renderer;
  gint position;
  GtkTreeViewColumn * col;  
  GtkTreeSelection   * list_selection;
  GtkTreeModel  * model;
  GtkWidget * update_button;
  
  viewer 		  = g_new(GTracerUSignalViewer, 1);

  model = gtracer_usignal_model_new(NULL); /* TODO */
  viewer->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_inspector_tag_to_widget (viewer->window);
  gtk_window_set_title(GTK_WINDOW(viewer->window),
		       _("Unix Signals"));
  g_signal_connect(G_OBJECT(viewer->window),
		   "delete_event",
		   G_CALLBACK(gtk_widget_hide_on_delete),
		   NULL);
  vbox = gtk_vbox_new(FALSE, 4);
  gtk_container_add(GTK_CONTAINER(viewer->window), vbox);
  
  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  adjustment 	  = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrolled_window));
  g_inspector_tag_to_object(G_OBJECT(adjustment));
  adjustment 	  = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolled_window));
  g_inspector_tag_to_object(G_OBJECT(adjustment));
  
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				 GTK_POLICY_AUTOMATIC, 
				 GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 2);

  viewer->view	= gtk_tree_view_new ();
  text_renderer = gtk_cell_renderer_text_new ();
  g_inspector_tag_to_object(G_OBJECT(text_renderer));

  position = 0;
  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(viewer->view),
							position,
							_("Number"),
							text_renderer,
							gtracer_usignal_model_extract_number,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(viewer->view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(viewer->view),
							position,
							_("Name"),
							text_renderer,
							gtracer_usignal_model_extract_name,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(viewer->view), position - 1);
  gtracer_register_tree_view_column(col);

  position = gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(viewer->view),
							position,
							_("Action"),
							text_renderer,
							gtracer_usignal_model_extract_action,
							NULL,
							NULL);
  col = gtk_tree_view_get_column(GTK_TREE_VIEW(viewer->view), position - 1);
  gtracer_register_tree_view_column(col);

  gtk_tree_view_set_model (GTK_TREE_VIEW (viewer->view), model);
  g_object_unref (G_OBJECT(model));

  gtk_container_add(GTK_CONTAINER(scrolled_window), viewer->view);
    
  list_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (viewer->view));
  g_inspector_tag_to_object(G_OBJECT(list_selection));

  gtk_tree_selection_set_mode (GTK_TREE_SELECTION (list_selection),
			       GTK_SELECTION_SINGLE);
  g_signal_connect (list_selection, 
		    "changed", 
		    G_CALLBACK (gtracer_usignal_viewer_selection_changed), 
		    viewer);

  gtk_widget_set_size_request (viewer->view, 350, 150);

  viewer->statusbar = gtk_statusbar_new();
  viewer->context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(viewer->statusbar),
						    "Unix signal extra information");
  /* Pushing a dummy string */
  gtk_statusbar_push(GTK_STATUSBAR(viewer->statusbar), 
		     viewer->context_id, "");
  
  gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(viewer->statusbar), FALSE);
  gtk_box_pack_start (GTK_BOX (vbox), viewer->statusbar,
		      FALSE, FALSE, 2);
  update_button = gtk_button_new_with_label(_("Update"));
  g_signal_connect_swapped(G_OBJECT(update_button), "clicked",
			   G_CALLBACK(gtracer_usignal_viewer_update),
			   viewer);
  gtk_box_pack_end(GTK_BOX(viewer->statusbar), update_button,
		   FALSE, FALSE, 2);

  return viewer;
}

static gpointer
gtracer_usignal_viewer_show(gpointer data, gpointer func_data, gpointer invoke_data)
{
  GTracerUSignalViewer * viewer = func_data;

  g_return_val_if_fail (viewer, NULL);
  g_return_val_if_fail (viewer->window, NULL);

  gtracer_usignal_viewer_update(viewer);
  gtk_widget_show_all(viewer->window);

  return viewer->window;
}

static void
gtracer_usignal_viewer_update(GTracerUSignalViewer * viewer)
{
  GtkTreeModel  * model;  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(viewer->view));
  gtracer_usignal_model_update(model, NULL); /* TODO */
}

static void
gtracer_usignal_viewer_selection_changed(GtkTreeSelection *selection,
					 gpointer user_data)
{
  GTracerUSignalViewer * viewer = user_data;
  GtkTreeModel * model;
  GtkTreeIter iter;
  gchar * msg;

  if (gtk_tree_selection_get_selected(selection, &model, &iter) == TRUE)
    {
      gtk_statusbar_pop(GTK_STATUSBAR(viewer->statusbar), viewer->context_id);
      msg = gtracer_usignal_model_extract_message(model,
						  &iter);
      if (msg) {
	gtk_statusbar_push(GTK_STATUSBAR(viewer->statusbar), 
			     viewer->context_id,
			     msg);
	g_free(msg);
      } else 
	gtk_statusbar_push(GTK_STATUSBAR(viewer->statusbar), 
			   viewer->context_id,
			   "");
    }
}
/* arch-tag: 2214fd8d-0d25-44f0-bc29-d28bd47ce5e
   (do not change this comment) */
