/* utils.c --- implementation of common miscellaneous functions

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "utils.h"
#include <ginspector/ginspector-gtk-tag.h>
#include <ginspector/ginspector-tag.h>

void
gtracer_register_tree_view_column (GtkTreeViewColumn * tree_view_column)
{
  g_return_if_fail (tree_view_column);

  g_inspector_tag_to_object(G_OBJECT(tree_view_column));
  if (tree_view_column->tree_view) 
    g_inspector_tag_to_widget(tree_view_column->tree_view);
  
  if (tree_view_column->button) 
    g_inspector_tag_to_widget(tree_view_column->button);

  if (tree_view_column->child) 
    g_inspector_tag_to_widget(tree_view_column->child);

  if (tree_view_column->arrow) 
    g_inspector_tag_to_widget(tree_view_column->arrow);

  if (tree_view_column->alignment) 
    g_inspector_tag_to_widget(tree_view_column->alignment);

  if (tree_view_column->window) 
    g_inspector_tag_to_object(G_OBJECT(tree_view_column->window));

  /* TODO: GtkCellEditable *editable_widget; */
}

struct _GTracerIterStack
{
  GSList * list;
};

GTracerIterStack*
gtracer_iter_stack_new(void)
{
  GTracerIterStack * stack;
  GtkTreeIter * iter;
  stack       = g_new(GTracerIterStack, 1);
  iter        = g_new(GtkTreeIter, 1);
  stack->list = NULL;

  stack->list = g_slist_prepend(stack->list, iter);
  return stack;
}

void
gtracer_iter_stack_clear(GTracerIterStack* stack)
{
  while (gtracer_iter_stack_parent_iter(stack)) 
    gtracer_iter_stack_pop(stack);
}
GtkTreeIter *
gtracer_iter_stack_current_iter(GTracerIterStack* stack)
{
  GSList * list = stack->list;
  g_return_val_if_fail (list, NULL);
  return list->data;
}

GtkTreeIter *
gtracer_iter_stack_parent_iter(GTracerIterStack* stack)
{
  GSList * list = stack->list;
  
  g_assert(list);
  
  if (list->next)
    return list->next->data;
  else
    return NULL;
}

void
gtracer_iter_stack_pop(GTracerIterStack* stack)
{
  GSList * list = stack->list;
  g_return_if_fail (list);

  g_free(list->data);
  stack->list = g_slist_delete_link(list, list);
}

GtkTreeIter *
gtracer_iter_stack_push_alloc(GTracerIterStack* stack)
{
  GtkTreeIter * iter;

  iter        = g_new(GtkTreeIter, 1);
  stack->list = g_slist_prepend(stack->list, iter);
  return iter;
}

gpointer
g_ptr_array_last(GPtrArray * array)
{
  g_return_val_if_fail (array, NULL);
  if (array->len == 0)
    return NULL;
  return g_ptr_array_index(array, (array->len - 1));
}

/* ------------------------------------------------------------------
   Temporary helper function for adding menu items to g-inspector's 
   main window.
   ------------------------------------------------------------------ */
extern void g_inspector_main_menu_append_item (gchar *item_path,
                                               gchar *item_type,
                                               gchar *func_name);

void
g_tracer_append_menu_item (gchar *item_path,
			   gchar *item_type,
			   gchar *func_name)
{
  g_inspector_main_menu_append_item(item_path, item_type, func_name);
}


/* arch-tag: 45a38f74-c557-49ac-a1f4-4c9e7db2f718
   (do not change this comment) */
