/* viewer.c --- implementation of viewers common routines

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "plugin.h"
#include <ginspector/ginspector-gtk-tag.h>

/* TODO: `gtracer_viewers' is not pluggable now.
   You cannot remove a viewer from `gtracer_viewers'.
 */

static GData * gtracer_viewers = NULL;

struct _GTracerViewer
{
  GQuark log_type;
  GTracerViewerFunc func;
  GTracerViewerDock dock;
  gpointer data;
  guint ref_count;
};


static GTracerViewer * gtracer_viewer_new  (GQuark log_type,
					    GTracerViewerFunc func,
					    GTracerViewerDock dock,
					    gpointer data);
static void            gtracer_viewer_free (GTracerViewer * viewer);


void
gtracer_viewers_init(void)
{
  if (gtracer_viewers == NULL)
    g_datalist_init(&gtracer_viewers);
}

GTracerViewer*
gtracer_viewers_find    (GQuark log_type)
{
  return g_datalist_id_get_data(&gtracer_viewers, log_type);
}

void
gtracer_viewers_install (const gchar * log_name,
			 GTracerViewerFunc viewer_func,
			 GTracerViewerDock viewer_dock,
			 gpointer viewer_data)
{
  GQuark log_type      	 = g_quark_from_string(log_name);
  GTracerViewer * viewer = gtracer_viewer_new(log_type, 
					      viewer_func, 
					      viewer_dock, 
					      viewer_data);
  g_datalist_id_set_data(&gtracer_viewers, log_type, viewer);
}


static GTracerViewer *
gtracer_viewer_new (GQuark log_type,
		    GTracerViewerFunc func,
		    GTracerViewerDock dock,
		    gpointer data)
{
  GTracerViewer * viewer = g_new(GTracerViewer, 1);

  viewer->log_type  = log_type;
  viewer->func 	    = func;
  viewer->dock	    = dock;
  viewer->data 	    = data;
  viewer->ref_count = 1;
  return viewer;
}

void
gtracer_viewer_push_log (GTracerViewer* viewer, 
			 GTracerLogProtected * log,
			 GTracerProxyLog * proxy_log)
{
  g_return_if_fail (viewer);
  g_return_if_fail (log);

  viewer->func(log, viewer->data, proxy_log);
}

GTracerViewer*
gtracer_viewer_ref   (GTracerViewer* viewer)
{
  g_return_val_if_fail (viewer, NULL);
  viewer->ref_count++;
  return viewer;
}

void
gtracer_viewer_unref (GTracerViewer* viewer)
{
  g_return_if_fail (viewer);
  viewer->ref_count--;
  if (viewer->ref_count == 0)
    gtracer_viewer_free(viewer);
}

static void
gtracer_viewer_free (GTracerViewer * viewer)
{
  g_return_if_fail (viewer);
  g_free(viewer);
}

void
gtracer_viewer_dock_filter_ui(GTracerViewer* viewer,
			      GtkWidget * filter_ui)
{
  if (viewer->dock) 
    {
      g_inspector_tag_to_widget (filter_ui);
      viewer->dock(filter_ui, viewer->log_type, viewer->data);
    }
}

/* arch-tag: a1735215-a202-4f5c-aa72-436c72090e45
   (do not change this comment) */
