/* pmaps.h --- Process memory mapping manipulator

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#ifndef __PMAPS_H__
#define __PMAPS_H__

#include <glib.h>
#include <stdio.h>

#include <sys/types.h>
#include <unistd.h>

typedef struct _PMapsEntry PMapsEntry;
typedef struct _PMaps PMaps;

typedef void (*PMapsForeachFunc)  (PMapsEntry * entry, gpointer callback_data);
typedef void (*PMapsUpdateNotify) (PMaps * pmaps, gboolean after, gpointer callback_data);

typedef enum 
{
  PMAPS_READABLE   = 1 << 0,
  PMAPS_WRITABLE   = 1 << 1,
  PMAPS_EXECUTABLE = 1 << 2,
  PMAPS_SHARED     = 1 << 3,	/* 0 means PRIVATE */
} PMapsProtection;

GQuark pmaps_errno_error_quark (void);
#define PMAPS_ERRNO_ERROR pmaps_errno_error_quark()

struct _PMapsEntry
{
  /* <public/> */
  
  /* Address range */
  guint          start;
  guint          end;

  /* Logical OR of PMapsProtection */
  guint          prot;

  /* Filename of library or executable */
  gchar*         filename;
  
  /* <private/> */
  GData*         attached_data;
  guint ref_count;
};

/*
 * PMaps functions
 */

PMaps*      pmaps_new     (pid_t pid, GError** err);
PMaps*      pmaps_ref     (PMaps* pmaps);
void        pmaps_unref   (PMaps* pmaps);

gboolean    pmaps_update  (PMaps* pmaps, GError** err);

PMapsEntry* pmaps_find    (PMaps* pmaps, guint address);
void        pmaps_foreach (PMaps* pmaps, 
			   PMapsForeachFunc func, 
			   gpointer callback_data);

void        pmaps_set_data (PMaps* pmaps,
			    const gchar* key,
			    gpointer data,
			    GDestroyNotify data_destroy_func);
gpointer    pmaps_get_data (PMaps* pmaps, const gchar* key);

gulong      pmaps_add_update_notify    (PMaps* pmaps, 
					PMapsUpdateNotify update_notify_func,
					gpointer callback_data);

void        pmaps_dump (PMaps * pmaps, const gchar* prefix);

/** pmaps_remove_update_notify
    returns TRUE if a hook assoicated with NOTIFY_ID is; and
    returns FALSE if no hook assoicated with NOTIFY_ID is. */
gboolean    pmaps_remove_update_notify (PMaps* pmaps, 
					gulong notify_id);


/*
 * Entry functions
 */

PMapsEntry* pmaps_entry_ref      (PMapsEntry* entry);
void        pmaps_entry_unref    (PMapsEntry* entry);


gint        pmaps_entry_include  (const PMapsEntry* entry, guint address);
void        pmaps_entry_set_data (PMapsEntry* entry,
				  const gchar* key,
				  gpointer data, 
				  GDestroyNotify data_destroy_func);
gpointer    pmaps_entry_get_data (PMapsEntry* entry, const gchar* key);

void        pmaps_entry_dump (PMapsEntry * entry, const gchar* prefix);

#endif  /* NOT: __PMAPS_H__ */

/* arch-tag: e937f8f7-d5c0-4c27-af1a-f14171cfcaca
   (do not change this comment) */
