/* functor.c --- Generator functions at runtime

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "functor.h"
#include <unistd.h>
#include <sys/mman.h>
#include <string.h>

static GTrashStack * functor_trash = NULL;

static gpointer functor_map_new (void);
static void     functor_trash_install_map(gpointer map);
static void     functor_init(gchar *f, 
			     GDestroyNotify original,
			     GTracerFunctorNotify cutin_after);

static void *
functor_map_new (void)
{
  return mmap(0, 
	      getpagesize(),
	      PROT_EXEC| PROT_WRITE,
	      MAP_PRIVATE| MAP_ANONYMOUS,
	      -1, 0);
}

static void
functor_trash_install_map(gpointer map)
{
  gchar * elt = map;
  while (elt < ((gchar *)map) + getpagesize()) 
    {
      g_trash_stack_push(&functor_trash, elt);
      elt = elt + gtracer_functor_size;
    }
}

GDestroyNotify
gtracer_functor_new(GDestroyNotify original,
		    GTracerFunctorNotify cutin_after)
{
  gpointer functor;

 retry:
  functor = g_trash_stack_pop(&functor_trash);
  if (functor == NULL)
    {    
      gpointer map = functor_map_new();
      functor_trash_install_map (map);
      goto retry;
    }
  
  functor_init(functor, original, cutin_after);
  return functor;

    
}

static void
functor_init(gchar *f, 
	     GDestroyNotify original,
	     GTracerFunctorNotify cutin_after)
{
  memset(f, 0, gtracer_functor_size);
  
  f[0x0] = 0x55;			       /* push   %ebp */
  f[0x1] = 0x89; f[0x2] = 0xe5;		       /* mov    %esp,%ebp  */
  f[0x3] = 0x83; f[0x4] = 0xec; f[0x5] = 0x14; /* sub    $0x14,%esp */
  f[0x6] = 0xff; f[0x7] = 0x75; f[0x8] = 0x08; /* pushl  0x8(%ebp) */

  f[0x9] = 0xe8;		/* call origin */
  *((guint *)&f[0xa]) = GPOINTER_TO_UINT(original) - (gint)(&(f[0xe]));

  f[0xe] = 0x68;		/* push `f' */
  *((unsigned *)&f[0xf]) = GPOINTER_TO_UINT(f);
  
  f[0x13] = 0x68;		/* push something unsigned */

  /* Free slot for unsigned: 0x14, 0x15, 0x16, 0x17 
     This slot is filled by `gtracer_functor_set_tag'. */
  f[0x14] = f[0x15] = f[0x16] = f[0x17] = 0x00;
  
  f[0x18] = 0xe8;		/* call cutin_after */
  *((guint *)&f[0x19]) = GPOINTER_TO_UINT(cutin_after) - (gint)(&(f[0x1d]));
  
  f[0x1d] = 0xc9;
  f[0x1e] = 0xc3;
}

void
gtracer_functor_set_tag(GDestroyNotify functor, guint tag)
{
  *(guint *)(((gchar*)functor) + 0x14) = tag;
}

void
gtracer_functor_free(gpointer functor)
{
  g_trash_stack_push(&functor_trash, functor);
}

/* arch-tag: 2e0f80b1-4677-409a-8853-88492c2de49f
   (do not change this comment) */

