/* init.c --- Initialize the preload 

   Copyright (c) 2004, 2005 by Masatake YAMATO and Red Hat, Inc.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.  

   - Neither the name of Red Hat, Inc nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGE. */

/* Development of the code in this file is support of 
   Information-technology Promotion Agency(IPA), Japan. */

#include "preload.h"
#include "gtracer.h"
#include <ginspector/ginspector-preload-manager.h>
#include <ginspector/ginspector-plugin-manager.h>

gboolean gtracer_preload_ready = FALSE;
static void gtracer_preload_start_tracing(void);

static GTracerPreloadOperations preload_operations = {
  .start_tracing = gtracer_preload_start_tracing,
};
static GTracerPreloadData preload_data = {
  .pops = &preload_operations,
};

static GInspectorPreloadModuleInfo  preload_info = {
  .name = GTRACER_PRELOAD_NAME,
  .data = &preload_data,
};

static void gtracer_preload_register (void);

static GTracerPluginOperations * gtracer_plugin_operations = NULL;
static GTracerPluginOperations * gtracer_preload_get_plugin_operations(void);

/* -------------------------------------------------------------------
   Preload module initializer
   ----------------------------------------------------------------- */
gboolean
gtk_init_check           (int    *argc,
			  char ***argv)
{
  static gboolean (* GENUINE) (int *argc, char ***argv) = NULL;
  gboolean result;

  GENUINE_GET();

  gtracer_preload_register();

  result = GENUINE(argc, argv);

  return result;
}

static void
gtracer_preload_register(void)
{
  static gboolean initialized = FALSE;

  if (initialized == TRUE)
    return ;
  initialized = TRUE;

  if (!g_thread_supported())
    g_thread_init (NULL);

  g_inspector_preload_register_module (&preload_info);
}

static void
gtracer_preload_start_tracing(void)
{
  g_assert(gtracer_preload_ready == FALSE);

  gtracer_preload_ready     = TRUE;
  gtracer_plugin_operations = gtracer_preload_get_plugin_operations();
}

static GTracerPluginOperations * 
gtracer_preload_get_plugin_operations(void)
{
  GInspectorPluginModuleInfo * plugin_info;
  GTracerPluginData * plugin_data;

  g_assert(gtracer_preload_ready == TRUE);

  plugin_info = g_inspector_plugin_find_module(GTRACER_PLUGIN_NAME);
  g_return_val_if_fail (plugin_info, NULL);
  
  plugin_data = plugin_info->data;
  g_return_val_if_fail (plugin_data, NULL);
  
  return plugin_data->mops;
}

/* -------------------------------------------------------------------
   Wrapper for function pointers provided from gtracer module 
   ----------------------------------------------------------------- */ 
gpointer
gtracer_preload_get_log_constructor(gchar * log_name)
{
  g_return_val_if_fail (log_name, NULL);
  g_return_val_if_fail (gtracer_plugin_operations, NULL);
  return gtracer_plugin_operations->get_log_constructor(log_name);
}

void
gtracer_preload_queue_push_log      (GTracerLog * log)
{
  g_return_if_fail (log);
  g_return_if_fail (gtracer_plugin_operations);
  gtracer_plugin_operations->push_log(log);
}

GTracerLog*
gtracer_preload_log_ref             (GTracerLog * log)
{
  g_return_val_if_fail (log, NULL);
  g_return_val_if_fail (log->lops, NULL);
  g_return_val_if_fail (log->lops->ref, NULL);
  log->lops->ref(log);
  return log;
}

void
gtracer_preload_log_unref           (GTracerLog * log)
{
  g_return_if_fail (log);
  g_return_if_fail (log->lops);
  g_return_if_fail (log->lops->unref);
  log->lops->unref(log);
}

/* ------------------------------------------------------------------
   Object life cycle management 
   ------------------------------------------------------------------ */
gboolean
gtracer_is_object_zombie(GObject * object)
{
  g_return_val_if_fail (object, FALSE);
  return (gboolean)g_object_get_data(object, GTRACER_OBJECT_ZOMBIE_KEY);
}

/* arch-tag: 47c35a20-0638-4b6a-a7ad-df1ba83ed85b
   (do not change this comment) */
